/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm;

import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import cin.dsadesktop.desktop.view.View;
import cin.dsadesktop.desktop.view.internalframes.BasicDesktopFrame;
import cin.dsadesktop.extension.DsaDesktopPluginExt;
import de.bernhardjung.dsaprogramm.basis.Kulturkunde;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.zauber.Zauber;

public class DsaDesktopPluginImpl implements DsaDesktopPluginExt {

    public String getAuthor() {
        return "Bernhard Jung";
    }

    public String getGroup() {
        return "Demo";
    }

	public ImageIcon getIconImage() {
		return Util.ladeIcon( "icon" );
	}

	public String getIcon() {
		return null;
	}


    public String getMail() {
        return "chargen@dsachargen.de";
    }

    public String getName() {
        return "DSA Charaktergenerator";
    }

    public BasicDesktopFrame getPluginFrame( final View hauptfenster ) {

        Basiswert.initialisieren();
        Eigenschaft.initialisieren();
        Talent.initialisieren();
        Kulturkunde.initialisieren();
        Sonderfertigkeit.initialisieren();
        VorNachteil.initialisieren();
        Zauber.initialisieren();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        try {

            speicher.ladeDaten();

            Rasse.LEERE_RASSE.setKulturen( speicher.getKulturen() );

        } catch ( final Exception e ) {

            JOptionPane.showMessageDialog( hauptfenster,
                    "Die Daten von Rassen, Waffen usw. konnten nicht geladen" +
                    " werden." );
        }

        final BasicDesktopFrame frame = new BasicDesktopFrame(
                I18n.i18n( "Programm.Name" ), true, true, true, true ) {};

        final Hauptpanel panel = new Hauptpanel( hauptfenster, frame, false );

        frame.setJMenuBar( panel.getMenue() );

        frame.add( panel );

        final Einstellungen einstellungen = Einstellungen.getInstanz();
        
        frame.setPreferredSize( 
                einstellungen.getHauptfensterGroesseDsaDesktop() );
        
        frame.addComponentListener( new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                einstellungen.setHauptfensterGroesseDsaDesktop( 
                        frame.getSize() );
            }
        } );

        return frame;
    }

    public String getTooltip() {
        return "Ein Charaktergenerator für die DSA Basisregeln";
    }
}
