/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.ATTR_BF;
import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.ATTR_ID;
import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.ATTR_INI;
import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.ATTR_NAME;
import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.ATTR_TYP;
import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.ATTR_WM_AT;
import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.ATTR_WM_PA;
import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.TAG_SCHILD;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

public class Schild
extends ElementMitNameImpl{
    
    /** Bruchfaktor */
    private int bf;

    /** Intiativemodifikator */
    private int ini;

    /** Die Groesse des Schildes als Fernkampfdeckung. */
    private SchildTyp typ;

    /** Attackemodifikator */
    private int wmAt;

    /** Parademodifikator */
    private int wmPa;

    /**
     * Erzeugt ein Dummy Schild, das verwendet werden kann, wenn kein 
     * eingegebenes Schild verfuegbar ist. Es sollte sonst nicht verwendet 
     * werden.
     */
    public Schild() {
        this( "SchildDummy" );
    }
    
    public Schild( final String dieId ) {
        super( dieId );

        bf = 3;

        ini = -1;

        typ = SchildTyp.GROSS;

        wmAt = -1;

        wmPa = 3;
    }

    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj ) {
            
            return false;
        }
        
        if ( ! getClass().equals( obj.getClass() ) ) {
            
            return false;
        }
        
        final Schild schild = (Schild) obj;
        
        return getId().equals( schild.getId() );
    }
    
    @Override
    public int hashCode() {
        
        return getId().hashCode();
    }
    
    /** @return den Bruchfaktor des Schildes. */
    public int getBf() {
        return bf;
    }

    /**
     * Aendert den Bruchfaktor.
     * @param bf Der neue Bruchfaktor.
     */
    public void setBf( final int bf ) {
        this.bf = bf;
    }

    /**
     * @return Der Initiativemodifikator.
     */
    public int getIni() {
        return ini;
    }

    /**
     * Aendert den Initiativemodifikator
     *
     * @param ini Der neue Initiativemodifikator.
     */
    public void setIni( final int ini ) {
        this.ini = ini;
    }

    /** @return die Groesse des Schildes als Fernkampfdeckung. */
    public SchildTyp getTyp() {
        return typ;
    }

    /** @param derTyp Die Groesse des Schildes als Fernkampfdeckung. */
    public void setTyp( final SchildTyp derTyp ) {
        typ = derTyp;
    }

    /** @return der Attackemodifikator. */
    public int getWmAt() {
        return wmAt;
    }

    /**
     * Aendert den Attackemodifikator.
     * @param wmAt Der neue Attackemodifikator.
     */
    public void setWmAt( final int wmAt ) {
        this.wmAt = wmAt;
    }

    /**
     * @return den Parademodifikator.
     */
    public int getWmPa() {
        return wmPa;
    }

    /**
     * @param wmPa der neue Parademodifikator
     */
    public void setWmPa( final int wmPa ) {
        this.wmPa = wmPa;
    }

    @Override
    public String toString() {
        return getName();
    }

    /**
     * @return ein XML-Element, das den Schild repraesentiert.
     */
    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_SCHILD );

        element.setAttribute( ATTR_BF, String.valueOf( bf ) );

        element.setAttribute( ATTR_ID, getId() );

        element.setAttribute( ATTR_INI, String.valueOf( ini ) );

        if ( null != name ) {

            element.setAttribute( ATTR_NAME, name );
        }

        element.setAttribute( ATTR_TYP, typ.getId() );

        element.setAttribute( ATTR_WM_AT, String.valueOf( wmAt ) );

        element.setAttribute( ATTR_WM_PA, String.valueOf( wmPa ) );

        return element;
    }
}
