/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

public interface SonderfertigkeitBonus {

    /**
     * Bietet alle Gelaendekunden zur Auswahl die ein Held noch nicht besitzt.
     */
    public static final SonderfertigkeitBonus GELAENDEKUNDE =
        new SonderfertigkeitBonusGelaendekunde();

    /**
     * Prueft ob ein Bonus eine bestimmte Sonderfertigkeit enthaelt.
     *
     * @param sonderfertigkeit
     *            die der Bonus enthalten soll.
     * @return {@code true}, wenn der Bonus die Sonderfertigkeit enthaelt, sonst
     *          {@code false}.
     */
    public boolean enthaelt( Sonderfertigkeit sonderfertigkeit );

    /**
     * @return die GP-Kosten, die einem Baustein, durch diesen Bonus entstehen.
     */
    public double getGpKosten();

    /**
     * @return die Liste der Werte von denen einer gewaehlt werden darf. Die
     *         Liste kann auch nur aus einem Element bestehen.
     */
    public List< SonderfertigkeitWert > getWerte( final Held held );

    /**
     * @return die XML-Repraesentation des Bonus.
     */
    public IXMLElement toXml();

}
