/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_SONDERFERTIGKEIT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_TALENT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VARIANTE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_WERT;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class SonderfertigkeitWertBonus {

    private final String variantenId;

    private final SonderfertigkeitWert wert;

    /**
     * @param derWert
     *            den man durch den Bonus erhaelt.
     * @param dieVariantenId
     *            die in der XML-Datei fuer die Variante der Sonderfertigkeit
     *            verwendet wird. Es kann auch einfach der Text der Variante
     *            uebergeben werden. Das Argument kann {@code null} sein.
     */
    public SonderfertigkeitWertBonus(
            final SonderfertigkeitWert derWert,
            final String dieVariantenId ) {

        wert = derWert;

        variantenId = dieVariantenId;
    }

    /**
     * Prueft ob es sich um einen Bonus auf eine bestimmte Sonderfertigkeit
     * handelt.
     *
     * @param sonderfertigkeit auf die geprueft werden soll.
     * @return {@code true}, wenn es sich um einen Bonus auf die
     *         sonderfertigkeit handelt, sonst {@code false}.
     */
    public final boolean enthaelt( final Sonderfertigkeit sonderfertigkeit ) {
        return wert.getSonderfertigkeit().equals( sonderfertigkeit );
    }

    /**
     * @return {@code true}, wenn die Sonderfertigkeit eine Variante besitzt,
     *         sonst {@code false}.
     */
    public final boolean hatVariantenId() {
        return null != variantenId;
    }

    /**
     * @return die Id, die in der XML-Datei fuer die Variante verwendet wird.
     */
    public final String getVariantenId() {
        return variantenId;
    }

    /**
     * @return den Wert, den man durch den Bonus erhaelt.
     */
    public final SonderfertigkeitWert getWert() {
        return wert.clone();
    }

    /** @return den Bonus als Text, zum Anzeigen. */
    @Override
    public final String toString() {
        return wert.toString();
    }

    /**
     * @return eine XML-Repraesentation des Bonus.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_WERT );

        final Sonderfertigkeit sonderfertigkeit = wert.getSonderfertigkeit();

        element.setAttribute( ATTR_SONDERFERTIGKEIT, sonderfertigkeit.getId() );

        final Talent talent = sonderfertigkeit.getTalent();

        if ( null != talent ) {

            element.setAttribute( ATTR_TALENT, talent.getId() );
        }

        if ( null != variantenId ) {

            element.setAttribute( ATTR_VARIANTE, variantenId );
        }

        return element;
    }
}
