/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_ALTE_SPRACHE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_BONUS;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListeFest;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;

public class TalentbonusFestAlteSprache extends TalentbonusFest {

    /** Der hoehere der beiden Boni, die vergeben werden. */
    private final int hoehererBonus;

    /** Der niedrigere der beiden Boni, die vergeben werden. */
    private final int niedrigererBonus;

    public TalentbonusFestAlteSprache(
            final List< Integer > dieStufenBoni) {

        super( new TalentListeFest( Arrays.asList( Talent.BOSPARANO,
                                          (Talent) Talent.UR_TULAMIDYA ) ),
                                    dieStufenBoni );

        final int bonus1 = dieStufenBoni.get( 0 );

        final int bonus2 = dieStufenBoni.get( 1 );

        if ( bonus1 < bonus2 ) {

            hoehererBonus = bonus2;

            niedrigererBonus = bonus1;

        } else {

            hoehererBonus = bonus1;

            niedrigererBonus = bonus2;

        }        
    }

    @Override
    public List< TalentWert > getTalentboni( final Held held ) {

        final List< TalentWert > boni = new ArrayList< TalentWert >( 2 );

        if ( held.hatMuttersprache( Talent.GARETHI ) ) {

            boni.add( new TalentWert( Talent.BOSPARANO, hoehererBonus ) );
            boni.add( new TalentWert( Talent.UR_TULAMIDYA, niedrigererBonus ) );

        } else if ( held.hatMuttersprache( Talent.TULAMIDYA ) ) {

            boni.add( new TalentWert( Talent.BOSPARANO, niedrigererBonus ) );
            boni.add( new TalentWert( Talent.UR_TULAMIDYA, hoehererBonus ) );

        } else {

            throw new IllegalArgumentException(
                    "Der Held hat weder Garethi noch Tulamidya als " +
                    "Muttersprache." );
        }

        return boni;
    }

    /**
     * @return {@code true}, falls der Held Garethi oder Tulamidya als
     *         Muttersprache hat, sonst {@code false}.
     */
    @Override
    public boolean istDirektAnwendbar( final Held held ) {
        return held.hatMuttersprache( Talent.GARETHI )
            || held.hatMuttersprache( Talent.TULAMIDYA );
    }

    @Override
    public String toString() {
        return String.format( 
                I18n.i18n( "TalentBonusFestAlteSprache" ), 
                hoehererBonus, 
                niedrigererBonus );
    }

    /** {@inheritDoc} */
    @Override
    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_ALTE_SPRACHE );

        for ( final int bonus : getBoni() ) {

            final IXMLElement bonusElement = new XMLElement( TAG_BONUS );

            bonusElement.setContent( String.valueOf( bonus ) );

            element.addChild( bonusElement );
        }

        return element;
    }
}
