/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.ungeeignete;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_AUSSER;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_VOR_NACHTEIL;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class UngeeigneterVorNachteilAusser implements UngeeigneterVorNachteil {

    /** Der Vor-/Nachteil der als ungeeignet gilt. */
    private final VorNachteil ungeeignet;

    /**
     * Besitzt der Held diesen Vor-/Nachteil, so gilt {@link #ungeeignet} nicht
     * als ungeeignet.
     */
    private final VorNachteil ausser;

    /**
     * @param derUngeeignete
     *            Vor-/Nachteil.
     * @param ausserDerHeldBesitztDiesen
     *            wenn der Held diesen Vor-/Nachteil besitzt gilt
     *            {@code der Ungeeignete} nicht als ungeeignet.
     */
    public UngeeigneterVorNachteilAusser(
            final VorNachteil derUngeeignete,
            final VorNachteil ausserDerHeldBesitztDiesen ) {

        ungeeignet = derUngeeignete;

        ausser = ausserDerHeldBesitztDiesen;

        ungeeignet.ergaenzeAbhaengigkeit( ausser );
    }

    public int compareTo(final UngeeigneterVorNachteil o) {
        return VorNachteil.NAME_COMPARATOR.compare( 
                ungeeignet, 
                o.getVorNachteil() );
    }

    /**
     * @return Der Vor-/Nachteil bei dem der Vor-/Nachteil nicht als ungeeignet
     *         gilt.
     */
    public VorNachteil getAusser() {
        return ausser;
    }

    /** {@inheritDoc} */
    public VorNachteil getVorNachteil() {
        return ungeeignet;
    }

    /** {@inheritDoc} */
    public boolean istUngeeignetFuer(final Held held) {
        return ! held.hat( ausser );
    }

    @Override
    public String toString() {
        return String.format( 
                I18n.i18n( "UngeeigneterVorNachteilAusser" ), 
                ungeeignet.getName(), 
                ausser.getName() );
    }

    /** {@inheritDoc} */
    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_VOR_NACHTEIL );

        element.setContent( ungeeignet.getId() );

        element.setAttribute( ATTR_AUSSER, ausser.getId() );

        return element;
    }
}
