/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys;

import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_ID;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_NAME;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_REICHWEITEN;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_TALENT;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_TP;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_TP_PLUS;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basis.Fernkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Tp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class FactoryFernwaffen extends Factory< Fernkampfwaffe > {

    @Override
    public final List< Fernkampfwaffe > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< Fernkampfwaffe > waffen = new ArrayList< Fernkampfwaffe >();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final IXMLElement waffeElement : wurzelElement.getChildren() ) {

            final String id = waffeElement.getAttribute( ATTR_ID );

            final Fernkampfwaffe waffe = speicher.getFernkampfwaffe( id );

            if ( waffeElement.hasAttribute( ATTR_NAME  ) ) {

                waffe.setName( waffeElement.getAttribute( ATTR_NAME ) );
            }

            waffe.setReichweiten(
                    getIntListe( waffeElement, ATTR_REICHWEITEN ) );

            final String talentId = waffeElement.getAttribute( ATTR_TALENT );

            waffe.setTalent( (Kampftechnik) Talent.getTalent( talentId ) );

            waffe.setTp( new Tp( waffeElement.getAttribute( ATTR_TP ) ) );

            waffe.setTpPlus(
                    getIntListe( waffeElement, ATTR_TP_PLUS ) );

            waffen.add( waffe );
        }

        return waffen;
    }
}
