/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BASISWERT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_EIGENSCHAFT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_MODIFIKATION;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_BASISWERT_MOD;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_EIGENSCHAFT_MOD;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_MODIFIKATIONEN;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.bausteine.util.BasiswertModifikation;
import de.bernhardjung.dsaprogramm.bausteine.util.EigenschaftsModifikation;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;

public class FactoryModifikationenIn {

    /**
     * @param wurzelElement des XML-Dokuments.
     * @return die aus dem XML-Dokument ermittelten Modifikationen.
     */
    public final Modifikationen erzeugen(
            final IXMLElement wurzelElement ) {

        final Modifikationen modifikationen = new Modifikationen();

        final IXMLElement modifikationenElement =
            wurzelElement.getFirstChildNamed( TAG_MODIFIKATIONEN );

        if ( null != modifikationenElement ) {

            for ( final IXMLElement element :
                modifikationenElement.getChildrenNamed( TAG_BASISWERT_MOD ) ) {

                final String id =
                    element.getAttribute( ATTR_BASISWERT );

                final Basiswert basiswert = Basiswert.getBasiswert( id );

                final String modifikationString =
                    element.getAttribute( ATTR_MODIFIKATION );

                final int modifikation = Integer.valueOf( modifikationString );

                modifikationen.basiswerte.add(
                        new BasiswertModifikation( basiswert, modifikation ) );
            }

            for ( final IXMLElement element :
                modifikationenElement.getChildrenNamed( TAG_EIGENSCHAFT_MOD ) ) {

                final String id =
                    element.getAttribute( ATTR_EIGENSCHAFT );

                final Eigenschaft eigenschaft;
                eigenschaft = Eigenschaft.getEigenschaft( id );

                final String modifikationString =
                    element.getAttribute( ATTR_MODIFIKATION );

                final int modifikation = Integer.valueOf( modifikationString );

                modifikationen.eigenschaften.add(
                        new EigenschaftsModifikation(
                                eigenschaft, modifikation ) );
            }
        }

        return modifikationen;
    }

    public class Modifikationen {

        public final List< BasiswertModifikation > basiswerte =
            new ArrayList< BasiswertModifikation >();

        public final List< EigenschaftsModifikation > eigenschaften =
            new ArrayList< EigenschaftsModifikation >();
    }
}
