/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_SONDERFERTIGKEIT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_TALENT;
import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class FactorySonderfertigkeitIn {

    /**
     * Ermittelt eine Sonderfertigkeit aus einem XML-Element. Das Element
     * benoetigt das Attribut Sonderfertigkeit, das die Id einer
     * Sonderfertigkeit enthaelt und ggf. das Attribut Talent, das die Id eines
     * Talents enthaelt.
     *
     * @param wertElement
     *            Das Element aus dem die Sonderfertigkeit ermittelt werden
     *            soll.
     * @return die ermittelte Sonderfertigkeit.
     */
    protected final Sonderfertigkeit ermitteleSonderfertigkeit(
            final IXMLElement wertElement ) {

        // Erst die Sonderfertigkeit feststellen.
        final String id =
            wertElement.getAttribute( ATTR_SONDERFERTIGKEIT );

        final Sonderfertigkeit sonderfertigkeit;

        if ( id.equals( Sonderfertigkeit.ID_SCHARFSCHUETZE ) ) {

            final String idTalent =
                wertElement.getAttribute( ATTR_TALENT );

            final Kampftechnik kampftechnik =
                (Kampftechnik) Talent.getTalent( idTalent );

            sonderfertigkeit =
                Sonderfertigkeit.getScharfschuetze( kampftechnik );

        } else if (
                id.equals( Sonderfertigkeit.ID_TALENTSPEZIALISIERUNG )){

            final String idTalent =
                wertElement.getAttribute( ATTR_TALENT );

            final Talent talent = Talent.getTalent( idTalent );

            sonderfertigkeit =
                Sonderfertigkeit.getTalentspezialisierung( talent );

        } else if (
                id.equals( Sonderfertigkeit.ID_WAFFENSPEZIALISIERUNG )){

            final String idTalent =
                wertElement.getAttribute( ATTR_TALENT );

            final Kampftechnik kampftechnik =
                (Kampftechnik) Talent.getTalent( idTalent );

            sonderfertigkeit =
                Sonderfertigkeit.getWaffenspezialisierung(
                        kampftechnik );

        } else {

            sonderfertigkeit =
                Sonderfertigkeit.getSonderfertigkeit( id );
        }

        return sonderfertigkeit;
    }
}
