package de.bernhardjung.dsaprogramm.gui;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.zauber.Repraesentation;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;
import de.bernhardjung.dsaprogramm.zauber.Zaubergruppe;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

public class PanelBenutzerdefinierterZauber extends JPanel {

    private static final String GRUPPE = "PanelBenutzerdefinierterZauber.Gruppe";

    private static final String NAME = "PanelBenutzerdefinierterZauber.Name";

    private static final String PROBE = "PanelBenutzerdefinierterZauber.Probe";

    private static final String REPRAESENTATION = "PanelBenutzerdefinierterZauber.Repraesentation";

    private static final String SPALTE = "PanelBenutzerdefinierterZauber.Spalte";

    private final JRadioButton gruppeBegabungenRadioButton;

    private final JRadioButton gruppeZauberRadioButton;

    private final JRadioButton repraesentationElfen;

    private final JRadioButton repraesentationMagier;

    /** Z.B. Armatrutz */
    private final JTextField name;

    /** Hier werden die drei Eigenschaften fuer die Probe eingegeben. */
    private final PanelEigenschaftsprobe probe;

    private final ComboBoxSteigerungsspalte spalte;

    /** Kann die Texte im Panel beim Sprachwechsel neu uebersetzen. */
    private final Uebersetzer uebersetzer;

    public PanelBenutzerdefinierterZauber() {

        uebersetzer = new Uebersetzer();

        gruppeBegabungenRadioButton = new JRadioButton(
                Zaubergruppe.BEGABUNG.getName() );
        gruppeZauberRadioButton = new JRadioButton(
                Zaubergruppe.ZAUBER.getName() );
        gruppeZauberRadioButton.setSelected( true );
        ButtonGroup group = new ButtonGroup();
        group.add( gruppeBegabungenRadioButton );
        group.add( gruppeZauberRadioButton );

        name = new JTextField();

        probe = new PanelEigenschaftsprobe();

        spalte = new ComboBoxSteigerungsspalte();

        repraesentationElfen = new JRadioButton(
                Repraesentation.ELFEN.getName() );
        repraesentationMagier = new JRadioButton(
                Repraesentation.MAGIER.getName() );
        repraesentationMagier.setSelected( true );
        group = new ButtonGroup();
        group.add( repraesentationElfen );
        group.add( repraesentationMagier );

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        FormLayout layout = new FormLayout( "P, 4dlu, P" );
        DefaultFormBuilder builder = new DefaultFormBuilder( layout, this );
        builder.setDefaultDialogBorder();

        builder.append( uebersetzer.erzeugeLabel( NAME ) );
        builder.append( name );

        builder.append( uebersetzer.erzeugeLabel( GRUPPE ) );
        builder.append( erzeugePanelGruppe() );

        builder.append( uebersetzer.erzeugeLabel( REPRAESENTATION ) );
        builder.append( erzeugePanelRepraesentation() );

        builder.append( uebersetzer.erzeugeLabel( PROBE ) );
        builder.append( probe );

        builder.append( uebersetzer.erzeugeLabel( SPALTE ) );
        builder.append( spalte );
    }

    private JPanel erzeugePanelGruppe() {

        FormLayout layout = new FormLayout( "P, 4dlu, P" );
        DefaultFormBuilder builder = new DefaultFormBuilder( layout  );

        builder.append( gruppeZauberRadioButton );
        builder.append( gruppeBegabungenRadioButton );

        return builder.getPanel();
    }

    private JPanel erzeugePanelRepraesentation() {

        FormLayout layout = new FormLayout( "P, 4dlu, P" );
        DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.append( repraesentationMagier );
        builder.append( repraesentationElfen );

        return builder.getPanel();
    }

    private List< Repraesentation > getRepraesentationen() {

        List< Repraesentation > reps = new ArrayList< Repraesentation >();
        if (repraesentationElfen.isSelected() ) {
            reps.add( Repraesentation.ELFEN );
        } else {
            reps.add( Repraesentation.MAGIER );
        }
        return reps;
    }

    /** Liefert die eingegebenen Daten. */
    private Variante getVariante() {

        String nameString = name.getText().trim();
        Variante variante = new Variante( nameString );
        variante.setProbe( probe.getProbe() );
        variante.setRepraesentationen( getRepraesentationen() );
        variante.setSteigerungsspalte( spalte.getGewaehlteSpalte() );
        return variante;
    }

    /**
     * Erzeugt einen Wert mit den eingegebenen Daten.
     * Der Wert kann dem helden hinzugefuegt werden.
     */
    public ZauberWert getWert() {

        Zauber zauber;
        if (gruppeBegabungenRadioButton.isSelected())
            zauber = Zauber.BENUTZERDEFINIERTE_BEGABUNG;
        else
            zauber = Zauber.BENUTZERDEFINIERTER_ZAUBER_;

        Variante variante = getVariante();

        return new ZauberWert( zauber, variante, false );
    }

}
