/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelTalente
extends ResetbaresPanel 
implements Observer, Uebersetzbar {

    /** Der Text neben der Combobox mit der man die Lernmethode waehlen kann. */
    private static final String LERNMETHODE =
        "PanelTalente.Lernmethode";

    /** Die maximal Anzahl an Talenten die aktiviert werden darf. */
    public static final int MAX_AKTIVIEREN = 5;

    /**
     * Ein Text der dem Benutzer mitteilt, dass er genau ein Talent aktiviert
     * hat.
     */
    private static final String TALENT_AKTIVIERT =
        "PanelTalente.TalentAktiviert";

    /**
     * Ein Text der dem Benutzer mitteilt wie viele Talent von ihm aktiviert
     * wurden.
     */
    private static final String TALENTE_AKTIVIERT =
        "PanelTalente.TalenteAktiviert";

    /**
     * Erklaert dem Benutzer, dass nicht mehr als 5 Talente aktivert werden
     * duerfen.
     */
    public static final String ZU_VIELE_AKTIVIERT =
        "PanelTalente.ZuVieleAktiviert";

    /**
     * Das Label mit dem angezeigt wird wie viele Talente der Benutzer schon
     * aktiviert hat.
     */
    private final JLabel aktivierteTalente;

    /** Der Held dessen Talente angezeigt und bearbeitet werden. */
    private Held held;
    
    /** Die Liste der Kampftechniken. */
    private final PanelKampftechnikenListe kampftechniken;

    /** Die Combobox mit der man die Lernmethode waehlt. */
    private final JComboBox lernmethode;
    
    /** 
     * Synchrnoisiert die in der Checkbox gewaehlte Lernmethode mit den Panels.
     */
    private final Lernmethoden lernmethoden;
    
    /**
     * Die Talente der einzelnen Talentgruppen mit Ausnahme der Talentgruppe
     * Kampf (siehe oben). Fuer jede Talentgruppe gibt es ein eigenes
     * PanelTalente.
     */
    private final Collection< PanelTalenteListe > talente;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Erzeugt ein neues Panel. */
    public PanelTalente( final boolean zumSteigern ) {

        uebersetzer = new Uebersetzer();
    
        aktivierteTalente = new JLabel( I18n.i18n( TALENT_AKTIVIERT ) );

        aktivierteTalente.setHorizontalAlignment( SwingConstants.CENTER );
        
        lernmethode = uebersetzer.erzeugeComboBox( Lernmethode.values() );
        resetter.einfuegen( lernmethode );
        
        lernmethoden = new Lernmethoden( lernmethode );

        kampftechniken =
            new PanelKampftechnikenListe( zumSteigern, lernmethoden );
        resetter.einfuegen( kampftechniken );
        
        talente = new ArrayList< PanelTalenteListe >();
        
        aufbauenBenutzeroberflaeche( zumSteigern );
    }

    /** Baut die Benutzeroberflaeche auf. */
    private void aufbauenBenutzeroberflaeche( final boolean zumSteigern ){

        final SimpleInternalFrame kampftechnikenFrame = 
            uebersetzer.erzeugeFrame( 
                Talentgruppe.KAMPF.getLokalisierungsschluesselName(),
                kampftechniken );
        
        final JLabel lernmethodeLabel = uebersetzer.erzeugeLabel( LERNMETHODE );
        
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab( 
                tabbedPane, 
                Talentgruppe.KAMPF.getLokalisierungsschluesselKuerzel(),
                kampftechnikenFrame );

        for ( final Talentgruppe gruppe : Talentgruppe.getTalentgruppenAlle() ) {

            if ( Talentgruppe.KAMPF == gruppe ) {
                // Fuer die Kampftechniken haben wir ein extra Panel verwendet.
                // siehe oben
                continue;
            }

            final PanelTalenteListe panelTalente =
                new PanelTalenteListe( gruppe, zumSteigern, lernmethoden );
            resetter.einfuegen( panelTalente );
            
            talente.add( panelTalente );

            final SimpleInternalFrame frame = uebersetzer.erzeugeFrame( 
                        gruppe.getLokalisierungsschluesselName(),
                        panelTalente );
            
            uebersetzer.erzeugeTab( 
                    tabbedPane,
                    gruppe.getLokalisierungsschluesselKuerzel(),
                    frame );
        }

        if ( zumSteigern ) {

            setLayout( new FormLayout(
                    "0px:G, P, 2dlu, P, 0px:G",
                    "2dlu, P, 2dlu, F:0px:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( lernmethodeLabel, cc.xy( 2, 2 ) );
            add( lernmethode, cc.xy( 4, 2 ) );

            add( tabbedPane, cc.xyw( 1, 4, 5 ) );

        } else {

            setLayout( new FormLayout( "F:0px:G", "2dlu, P, 2dlu, F:0px:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( aktivierteTalente, cc.xy( 1, 2 ) );

            add( tabbedPane, cc.xy( 1, 4 ) );
        }
    }
    
    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
        
        update( null, null );
    }
    
    /**
     * Bestimmt den Helden dessen Werte angezeigt werden.
     *
     * @param derHeld dessen Werte angezeigt werden sollen.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;
        
        held.addObserver( this );

        kampftechniken.setHeld( held );

        for ( final PanelTalenteListe panel : talente ) {

            panel.setHeld( held );

        }

        update( null, null );
    }

    /**
     * Wir aufgerufen wenn am ueberwachten Helden eine Veraenderung
     * durchgefuehrt wird.
     *
     * @param o ungenutzt.
     * @param arg ungenutzt.
     */
    public final void update( final Observable o, final Object arg ) {

        final int anzahlAktivierteTalente = held.getAnzahlAktivierteTalente();

        final String text;

        if ( 1 == anzahlAktivierteTalente ) {

            text = I18n.i18n( TALENT_AKTIVIERT );

        } else {

            text = String.format( 
                    I18n.i18n( TALENTE_AKTIVIERT ), 
                    anzahlAktivierteTalente );
        }

        if ( anzahlAktivierteTalente > MAX_AKTIVIEREN ) {

            aktivierteTalente.setForeground( Util.FEHLERFARBE );
            aktivierteTalente.setToolTipText(
                    "<html>" + I18n.i18n( ZU_VIELE_AKTIVIERT ) );

        } else {

            aktivierteTalente.setForeground( getForeground() );
            aktivierteTalente.setToolTipText( null );

        }

        aktivierteTalente.setText( text );
    }
}
