/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class PanelVerbindungenListe
extends ResetbaresPanel 
implements Observer, Uebersetzbar {

    /** Die Ueberschrift fuer die Spalte mit den Spinnern mit der Gesamtstufe. */
    private static final String GESAMT =
        "PanelVerbindungenListe.Gesamt";

    /** Die Ueberschrift fuer die Spalte mit den Kosten. */
    private static final String KOSTEN =
        "PanelVerbindungenListe.Kosten";

    /** Zeigt dem Benutzer an wie viele Punkte noch zu verteilen sind. */
    private static final String REST =
        "PanelVerbindungenListe.Rest";

    /** Zeigt dem Benutzer an, dass nur noch 1 Punkt zu verteilen ist. */
    private static final String REST_EIN_PUNKT =
        "PanelVerbindungenListe.RestEinPunkt";

    /** Die Ueberschrift fuer die Spalte mit den verteilten Punkten. */
    private static final String VERTEILT = "PanelVerbindungenListe.Verteilt";

    /** Ein String um die GP-Kosten anzuzeigen, z.B. "%1$s GP" */
    private static final String GP_KOSTEN = "PanelVerbindungenListe.GpKosten";

    /** Wird aufgerufen wenn eine Verbindung (ab)gewaehlt wird. */
    private final ActionListener actionDeAktivieren;

    /** Wird aufgerufen wenn die Variante eines Wertes geaendert wird. */
    private final ActionListener actionVariante;

    /**
     * Speichert fuer jede Combobox fuer welchen Wert in {@link #werte} sie
     * zustaendig ist.
     */
    private final Map< JComboBox, Integer > comboboxIndex;

    /**
     * Die Spinner die anzeigen wie hoch die Stufe des Helden in der Verbindung
     * insgesamt ist. Die Gesamtstufe sind die Punkte die man aus der Profession
     * erhalten hat plus die Punkte die man noch ueber GP dazugekauft hat.
     */
    private final List< JSpinner > gesamtstufe;

    /**
     * Die Ueberschrift fuer die Spalte in der die Stufe der Verbindung in den
     * Spinnern angezeigt und eingestellt wird.
     */
    private final JLabel gesamtUeberschrift;
    
    /** Der Held fuer den die Verbindungen gewaehlt werden. */
    private Held held;

    /**
     * Zeigt die Kosten fuer die Punkte an, die man noch durch GP dazu gekauft
     * hat.
     */
    private final List< JLabel > kosten;
    
    /**
     * Die Ueberschrift fuer die Spalte in der die GP Kosten der Verbindung
     * angezeigt werden.
     */
    private final JLabel kostenUeberschrift;

    /**
     * Wird aufgerufen wenn man die Gesamtstufe einer Verbindunge veraendert.
     */
    private final ChangeListener listenerGesamt;

    /**
     * Wird aufgerufen wenn man die Punkte veraendert die aus der Profession
     * auf die Verbindung verteilt werden.
     */
    private final ChangeListener listenerVerteilt;

    /**
     * Dieses Label zeigt dem Benutzer an wie viele Punkte noch zu verteilen
     * sind.
     */
    private final JLabel lRest;
    
    
    
    /**
     * Speichert fuer jeden Spinner fuer welchen Wert in {@link #werte} er
     * zustaendig ist.
     */
    private final Map< JSpinner, Integer > spinnerIndex;

    /** Uebersetzt die Ueberschriften der Liste. */
    private Uebersetzer uebersetzer;
    
    /** Die Comboboxen fuer die Variante der Verbindungen. */
    private final List< JComboBox > varianten;

    /** Die Checkboxen zum (de)aktivieren der Verbindungen. */
    private final List< JCheckBox > verbindungen;

    /**
     * Dieses Panel enthaelt die Liste mit den Verbindungen.
     */
    private final JPanel verbindungenListe;

    /**
     * Die Spinner mit denen die Punkte verteilt die man aus der Profession
     * erhaelt.
     */
    private final List< JSpinner > verteilt;

    /**
     * Die Ueberschrift fuer die Spalte in der die Punkte verteilt werden, die
     * man durch die Profession erhalten hat.
     */
    private final JLabel verteiltUeberschrift;
    
    /** Die Werte die in der Liste angezeigt werden. */
    private final List< VorNachteilWert > werte;

    /**
     * Die Punkte die insgesamt auf die Verbindungen verteilt werden duerfen.
     */
    private int zuVerteilendePunkte;

    PanelVerbindungenListe() {

        uebersetzer = new Uebersetzer();
        
        
        
        actionDeAktivieren = new ActionDeAktivieren();

        actionVariante = new ActionVariante();

        comboboxIndex = new HashMap<JComboBox, Integer>();

        gesamtstufe = new ArrayList<JSpinner>();

        gesamtUeberschrift = uebersetzer.erzeugeUeberschrift( GESAMT );
        
        kosten = new ArrayList<JLabel>();

        kostenUeberschrift = uebersetzer.erzeugeUeberschrift( KOSTEN );
        
        listenerGesamt = new GesamtListener();

        listenerVerteilt = new VerteiltListener();

        lRest = new JLabel( "noch 25 Punkte zu verteilen" );
        lRest.setHorizontalAlignment( JLabel.CENTER );

        spinnerIndex = new HashMap<JSpinner, Integer>();

        varianten = new ArrayList<JComboBox>();

        verbindungen = new ArrayList<JCheckBox>();

        verbindungenListe = new JPanel();

        verteilt = new ArrayList<JSpinner>();

        verteiltUeberschrift = uebersetzer.erzeugeUeberschrift( VERTEILT );
        
        werte = new ArrayList<VorNachteilWert>();

        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( verbindungenListe );
        resetter.einfuegen( scrollPane );
        
        setLayout( new FormLayout( "F:P:G",
                                   "2dlu, P, 3dlu, F:P:G" ) );

        final CellConstraints cc = new CellConstraints();

        add( lRest, cc.xy( 1, 2 ) );

        add( scrollPane, cc.xy( 1, 4 ) );
    }

    /* *
     * Aendert die Variante eines Wertes.
     *
     * @param von Der Wert der geaendert werden soll.
     * @param nach Die Variante zu der der Wert geaendert werden soll.
     * /
    public final void aendereVariante(
            final VorNachteilWert von,
            final VorNachteilWert nach ) {

        if ( ! werte.contains( nach ) ) {

            final int index = werte.indexOf( von );

            if ( index != -1 ) {

                werte.get( index ).setVariante( nach.getVariante() );
            }
        }
    }
*/
    /**
     * Baut die Benutzeroberflaeche auf. Erzeugt alle Kompnenten und ordnet sie
     * auf dem Panel an.
     */
    private void aufbauenBenutzeroberflaeche() {

        verbindungenListe.removeAll();

        erzeugeElemente();
        
        verbindungenListe.setLayout( erzeugeLayout() );


        final CellConstraints cc = new CellConstraints();

        verbindungenListe.add( verteiltUeberschrift, cc.xy( 6 , 2 ) );

        verbindungenListe.add( gesamtUeberschrift, cc.xy( 8 , 2 ) );

        verbindungenListe.add( kostenUeberschrift, cc.xy( 10 , 2 ) );

        // in jeder 5. Zeile wird ein Trennstrich eingefuegt.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        // trennstriche gibt an wie viele Trennstriche schon eingefuegt wurden.
        for ( int i = 0, n = werte.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            int x = 2;
            final int y = ( 2 * i ) + 4;

            // alle 5 Zeilen ein Trennstrich.
            if ( ( ( i % 5 ) == 0 ) ) {

                verbindungenListe.add( new JSeparator(), cc.xyw( x, y, 9 ) );

                ++trennstriche;

                continue;
            }

            final int index = i - trennstriche;

            final JCheckBox checkBox = verbindungen.get( index );
            
            verbindungenListe.add( checkBox, cc.xy( x, y ) );

            
            final JComboBox variante = varianten.get( index );

            x += 2;
            verbindungenListe.add( variante, cc.xy( x, y ) );


            final JSpinner vert = verteilt.get( index );

            x += 2;
            verbindungenListe.add( vert, cc.xy( x, y ) );


            final JSpinner gesamt = gesamtstufe.get( index );

            x += 2;
            verbindungenListe.add( gesamt, cc.xy( x, y ) );


            final JLabel kosten = this.kosten.get( index );

            x += 2;
            verbindungenListe.add( kosten, cc.xy( x, y ) );
        }

    }

    private void erzeugeElemente() {
        
        for ( int i = gesamtstufe.size(), n = werte.size(); i < n ; ++i ) {
            
            final VorNachteil vorNachteil = werte.get( i ).getVorNachteil();
            
            final JCheckBox checkBox = uebersetzer.erzeugeCheckBox( 
                    vorNachteil.getL10nschluesselPraefix() );

            checkBox.addActionListener( actionDeAktivieren );

            // Jede Checkbox kennt den Index des Wertes fuer den sie da ist.
            checkBox.setActionCommand( String.valueOf( i ) );

            verbindungen.add( checkBox );


            final CharGenListModel< Variante, Variante > model =
                new CharGenListModel< Variante, Variante >(
                        Variante.NAME_COMPARATOR );

            final JComboBox variante = uebersetzer.erzeugeComboBox( model );
            resetter.einfuegen( variante );
            
            variante.setEditable( true );

            variante.addActionListener( actionVariante );

            comboboxIndex.put( variante, i );

            varianten.add( variante );

            
            final JSpinner vert = Util.erzeugeSpinner( 1, 0, 15, 1 );
            
            vert.addChangeListener( listenerVerteilt );

            spinnerIndex.put( vert, i );

            verteilt.add( vert );
            
            
            final JSpinner gesamt = Util.erzeugeSpinner( 1, 0, 15, 1 );

            gesamt.addChangeListener( listenerGesamt );

            spinnerIndex.put( gesamt, i );

            gesamtstufe.add( gesamt );
            
            
            final JLabel kosten = new JLabel( "0 GP" );

            this.kosten.add( kosten );
        }
    }
    
    /**
     * @return das Layout fuer die Benutzeroberflaeche.
     */
    private FormLayout erzeugeLayout() {
        final String spaltenLayout;
        //               Verbindungen  Variante    Stufe    Gesamt   Kosten
        spaltenLayout = "7dlu, P, 4dlu, F:P:G, 4dlu, P, 4dlu, P, 4dlu, P, 7dlu";

        // Jetzt das Zeilenlayout festlegen.
        // eine Zeile pro Verbindungen
        // und alle 5 Zeilen wird ein Trennstrich eingefuegt.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        final StringBuilder zeilenLayout = new StringBuilder();

        zeilenLayout.append( "7dlu, P," );

        for ( int i = werte.size() + extraZeilen; i > 0 ; --i ) {

            zeilenLayout.append( " 4dlu, P," );
        }

        zeilenLayout.append( " 7dlu" );

        return new FormLayout( spaltenLayout,
                               zeilenLayout.toString() );

    }

    /**
     * @return die restlichen Punkte, die noch verteilt werden duerfen.
     */
    private int getRestlichePunkte() {

        int verteiltePunkte = 0;

        for ( final VorNachteilWert wert : werte ) {

            if ( held.hatVerbindungen( wert ) ) {

                verteiltePunkte += wert.getStufe();
            }
        }

        return ( zuVerteilendePunkte - verteiltePunkte );
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
        
        final int rest = getRestlichePunkte();

        if ( 1 == rest ) {

            lRest.setText( I18n.i18n( REST_EIN_PUNKT ) );

        } else {

            lRest.setText( String.format( 
                    I18n.i18n( REST ), 
                    rest ) );

        }
    }
    
    /**
     * Bestimmt den Manager und damit den Held fuer die aktuelle Erschaffung.
     *
     * @param manager fuer die aktuelle Erschaffung.
     */
    public final void setManager( final ErschaffungManager manager ) {

        reset();
        
        held = manager.getHeld();

        held.addObserver( this );

        zuVerteilendePunkte = manager.getVerbindungen();

        werte.clear();

        for ( int i = zuVerteilendePunkte; i > 0 ; --i  ) {

            werte.add( new VorNachteilWert( 
                    VorNachteil.VERBINDUNGEN,
                    new Variante() ) );
        }

        aufbauenBenutzeroberflaeche();

        setSpinnerMaxima();

        update( null, VorNachteil.VERBINDUNGEN );
    }

    /**
     * Stellt das Maximum eines Spinners ein.
     * @param spinner dessen Maximum eingestellt werden soll
     * @param maximum fuer den Spinner.
     */
    private void setMaximum( final JSpinner spinner, final int maximum ) {

        final SpinnerNumberModel model;
        model = (SpinnerNumberModel) spinner.getModel();

        model.setMaximum( maximum );
    }

    /**
     * Stellt das Maximum fuer die Spinner ein, mit denen man die Punkte
     * verteilt.
     */
    private void setSpinnerMaxima() {

        final int rest = getRestlichePunkte();

        for ( int i = werte.size()-1; i >= 0 ; --i ) {

            final VorNachteilWert wert = werte.get( i );

            final int maximum;

            if ( held.hatVerbindungen( wert ) ) {

                maximum = Math.min( 15, wert.getStufe() + rest );

            } else {

                maximum = Math.min( 15,  rest );
            }

            final JSpinner spinner = verteilt.get( i );

            if ( (Integer) spinner.getValue() > maximum ) {

                spinner.setValue( maximum );
            }

            setMaximum( spinner, maximum );
        }
    }

    /**
     * Aktualisiert eine Zeile in der Benutzeroberflaeche.
     *
     * @param index des zu aktualisierenden Zeile.
     */
    @SuppressWarnings("unchecked")
    private void update( final int index ) {

        final VorNachteilWert wert = werte.get( index );

        final JCheckBox checkBox = verbindungen.get( index );

        if ( held.hatVerbindungen( wert ) ) {

            checkBox.setEnabled( true );

            checkBox.setSelected( true );

        } else if ( wert.getVariante().isNull()
                 || ( getRestlichePunkte() == 0 ) ) {

            checkBox.setEnabled( false );

            checkBox.setSelected( false );

        } else {

            checkBox.setEnabled( true );

            checkBox.setSelected( false );

        }

        final KonfliktBeschreibung beschreibung;

        if ( held.hat( wert ) ) {

            beschreibung = held.getWert( wert ).pruefen( held );

        } else {

            beschreibung = wert.pruefen( held );

        }

        if ( beschreibung.enthaeltKonflikt() ) {

            checkBox.setForeground( Util.FEHLERFARBE );
            checkBox.setToolTipText( beschreibung.toHtml() );

        } else {

            checkBox.setForeground( getForeground() );
            checkBox.setToolTipText( null );

        }

        final JComboBox comboBox = varianten.get( index );

        final VorNachteil vorNachteil = wert.getVorNachteil();

        final CharGenListModel< Variante, Variante > model =
            (CharGenListModel< Variante, Variante >) comboBox.getModel();

        final Variante variante = wert.getVariante();

        model.setElements( vorNachteil.getVarianten( held ), variante );

        final JSpinner vert = verteilt.get( index );

        if ( held.hatVerbindungen( wert ) ) {

            final int stufe = held.getVerbindungen( wert ).getStufe();

            wert.setStufe( stufe );
        }

        vert.setValue( wert.getStufe() );


        final JSpinner gesamt = gesamtstufe.get( index );

        if ( held.hat( wert ) ) {

            gesamt.setEnabled( true );

            gesamt.setValue( held.getWert( wert ).getStufe() );

        } else {

            gesamt.setEnabled( false );

            gesamt.setValue( wert.getStufe() );

        }


        final JLabel gp = kosten.get( index );

        if ( held.hat( wert ) ) {

            final int kosten = held.getWert( wert ).getGpKosten( held );

            gp.setText( String.format( I18n.i18n( GP_KOSTEN ), kosten ) );

        } else {

            gp.setText( String.format( I18n.i18n( GP_KOSTEN ), 0 ) );

        }
    }

    /**
     * Wird aufgerufen, wenn eine Veraenderung am Helden stattgefunden hat.
     *
     * @param o ungenutzt.
     * @param regelobjekt Das Regelobjekt, das geaendert wurde.
     */
    public void update( final Observable o, final Object regelobjekt ) {

        if ( VorNachteil.VERBINDUNGEN.equals( regelobjekt )
          || Eigenschaft.SO.equals( regelobjekt ) ) {

            final int rest = getRestlichePunkte();

            if ( 1 == rest ) {

                lRest.setText( I18n.i18n( REST_EIN_PUNKT ) );

            } else {

                lRest.setText( String.format( I18n.i18n( REST ), rest ) );
            }

            for ( int i = werte.size()-1; i >= 0; --i ) {

                update( i );
            }
        }
    }

    /**
     * Wird aufgerufen wenn auf eine Checkbox geklickt wird um eine Verbindung
     * an-/abzuwaehlen.
     *
     * @author Bernhard Jung
     */
    private class ActionDeAktivieren implements ActionListener {

        public final void actionPerformed( final ActionEvent e ) {

            final int index = Integer.valueOf( e.getActionCommand() );

            final VorNachteilWert wert = werte.get( index );

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            if ( checkBox.isSelected() ) {

                held.einfuegenVerbindungen( wert );

            } else {

                held.entfernenVerbindungen( wert );

            }

            setSpinnerMaxima();
        }
    }

    /**
     * Wird aufgerufen wenn die Variante eines Vorteils geaendert wird.
     *
     * @author Bernhard Jung
     */
    private class ActionVariante implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ( (String)variante ).trim();
            }

            final int index = comboboxIndex.get( comboBox );

            final VorNachteilWert wert = werte.get( index );

            wert.setVariante( variante );
            
            update( index );
            /*
            // Der testwert ist da um mit held.hat( testwert ) zu pruefen ob
            // der Held den Wert in dieser Variante schon besitzt.
            final VorNachteilWert testwert = wert.clone();

            testwert.setHeld( null );

            testwert.setVariante( variante );

            // nur wenn eine neue Variante gewaehlt wurde, ...
            if ( ! wert.getVariante().equals( variante )
                    // ... es eine sinnvolle Variante ist ...
                    && ( ! variante.equals( "" )
                      // oder der Wert nicht gewaehlt ist, ...
                      || ! held.hatVerbindungen( wert ) )
                    // ... und der Wert noch nicht vorgegeben ist.
                    && ( ! held.hatVorgegeben( testwert ) ) ) {

                if ( werte.contains( testwert ) ) {

                    final int i = werte.indexOf( testwert );

                    werte.get( i ).setVariante( "" );
                }

                final VorNachteil vorNachteil = wert.getVorNachteil();

                if ( vorNachteil.hatFreieVarianten() ) {

                    vorNachteil.einfuegenVariante( variante.toString() );
                }

                if ( ( ! held.hat( testwert ) )
                        && held.hatVerbindungen( wert ) ) {

                    held.setVerbindungenVariante( wert, variante );

                } else if ( held.hat( testwert )
                        &&  held.hatVerbindungen( wert ) ) {

                    held.getVorgegeben( wert ).setVariante( variante );

                    held.getVerbindungen( wert ).setVariante( variante );

                    final VorNachteilWert heldenwert = held.getWert( testwert );

                    if ( heldenwert.getStufe() < testwert.getStufe() ) {

                        heldenwert.setStufe( testwert.getStufe() );
                    }

                }
                
                wert.setVariante( variante );

            }

            update( index );
            */
        }
    }

    /**
     * Wird aufgerufen wenn der Wert eines Spinners geaendert wird, der die
     * Gesamtstufe anzeigt.
     *
     * @author Bernhard Jung
     *
     */
    private final class GesamtListener implements ChangeListener {
        /**
         * Wird aufgerufen wenn der Wert in einem der Spinner veraendert wird.
         */
        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = spinnerIndex.get( spinner );

            final VorNachteilWert wert = werte.get( i );

            final int stufe = (Integer) spinner.getValue();

            if ( held.hatVerbindungen( wert ) ) {

                final int verteilteStufe = held.getVerbindungen( wert ).getStufe();

                if ( verteilteStufe > stufe ) {

                    wert.setStufe( stufe );

                    held.setVerbindungenStufe( wert );

                    setSpinnerMaxima();

                } else {

                    held.getWert( wert ).setStufe( stufe );

                }
            }
        }
    }

    /**
     * Wird aufgerufen wenn der Wert eines Spinners, der die Punkte anzeigt die
     * aus der Profession kommen, geaendert wird.
     *
     * @author Bernhard Jung
     *
     */
    private final class VerteiltListener implements ChangeListener {
        /**
         * Wird aufgerufen wenn der Wert in einem der Spinner veraendert wird.
         */
        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = spinnerIndex.get( spinner );

            final VorNachteilWert wert = werte.get( i );

            final int stufe = (Integer) spinner.getValue();

            wert.setStufe( stufe );

            if ( held.hatVerbindungen( wert ) ) {

                held.setVerbindungenStufe( wert );

                setSpinnerMaxima();

            } else {

                update( i );

            }
        }
    }

}
