/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.action;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;

import de.bernhardjung.dsaprogramm.basis.heldio.FactoryHeldXmlOut;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class ActionSpeichernHeldXml extends ActionSpeichernHeld {

    private static final String FEHLER =
        "ActionSpeichernHeldXml.Fehler";

    private final FactoryHeldXmlOut factory;

    public ActionSpeichernHeldXml( final Hauptpanel hauptpanel  ) {

        super( hauptpanel,
                "ActionSpeichernHeldXml.Name",
                "speichern",
                "ActionSpeichernHeldXml.Mnemonic",
                "ActionSpeichernHeldXml.Tasten" );

        factory = new FactoryHeldXmlOut();
    }

    /**
     * @return '.dcg'
     */
    @Override
    String getDateierweiterung() {
        return ".dcg";
    }

    @Override
    String getVerzeichnis() {
        return Einstellungen.getInstanz().getHeldenXmlVerzeichnis();
    }

    @Override
    void setVerzeichnis( final File verzeichnis ) {

        final String pfad = verzeichnis.getAbsolutePath();

        Einstellungen.getInstanz().setHeldenXmlVerzeichnis( pfad );
    }

    @Override
    boolean speichern( final File datei ) {

        final String xmlDaten = factory.erzeugen( getHeld() );

        Writer out = null;

        try {

            out = new PrintWriter( datei, "UTF-8" );

            out.write( xmlDaten );

        } catch (final IOException e) {

            DialogFehlermeldung.zeigeDialog( I18n.i18n( FEHLER ), e );
            
            return false;

        } finally {

            if ( null != out ) {

                try {

                    out.close();

                } catch (final IOException e) {
                    
                    DialogFehlermeldung.zeigeDialog( I18n.i18n( FEHLER ), e );
                    
                    return false;
                }
            }
        }
        
        return true;
    }
    
    @Override
    void speichernErfolgreich() {
        
        hauptpanel.getPanelSteigern().heldGespeichert();
    }

}
