/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.bonus;

import java.awt.BorderLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelTalentBonusFest
extends PanelTalentBonus 
implements ItemListener {

    /**
     * Die Koordinaten des RadioButtons, der als letztes abgewaehlt wurde.
     * @see #itemStateChanged(ItemEvent)
     * @see #getKoordinaten(JRadioButton)
     */
    private int[] abgewaehlt = null;

    /**
     * Zeigen den aktuellen Talentwert des Helden an.
     */
    private final List< JLabel > aktuell;
    
    /**
     * Die Ueberschrift fuer die Spalte in der der aktuelle TaW des Helde
     * angezeigt wird.
     */
    private final JLabel aktuellUeberschrift;

    /**
     * Der Held fuer den der Bonus gewaehlt wird.
     */
    private Held held;
    
    /**
     * Eine Markierung die angibt ob gerade ob die gewaehlten RadioButtons
     * korrigiert werden, wenn der Benutzer einen Button in einer Zeile
     * angewaehlt hat, in der schon ein anderer Button angewaehlt ist.
     *
     * @see #itemStateChanged(ItemEvent)
     */
    private boolean korrektur = false;

    /**
     * Ein 2-dimensonales Feld mit den RadioButtons, die in der
     * Benutzeroberflaeche angezeigt werden.
     * An Position (y, x) im Feld seht der Button, der sich in der
     * Benutzeroberflaeche in Zeile y und Spalte x befindet.
     * Es werden dabei nur die Spalten und Zeilen mit Radio Buttons gerechnet.
     */
    private JRadioButton[][] radioButtons = new JRadioButton[ 0 ][ 0 ];

    /** Die Stufenboni die es zu verteilen gilt. */
    private List< Integer > stufenboni = new ArrayList< Integer >();

    /** Die Talente auf die es die Stufenboni zu verteilen gilt. */
    private List< Talent > talente = new ArrayList< Talent >();

    /**
     * Die Label in der erste Spalte, die die Namen der Talente anzeigen.
     */
    private List< JLabel > talenteLabel = new ArrayList< JLabel >();
    
    /** Erzeugt ein neues, leeres Panel. */
    public PanelTalentBonusFest() {
        
        aktuell = new ArrayList< JLabel >();
        
        aktuellUeberschrift = Util.erzeugeUeberschrift( 
                I18n.i18n( "PanelTalentBonusFest.aktuell" ) );
        
        setLayout( new BorderLayout() );
    }

    /**
     * Baut die Benutzerobeflaeche auf, wenn ein neuer Bonus angezeigt werden
     * soll. Die Liste der Talente wird angezeigt, die aktuellen Stufen der
     * Talente und die RadioButtons zum waehlen der Stufen-Boni.
     */
    private void aufbauenBenutzeroberflaeche() {

        removeAll();

        // pruefen ob im Feld genuegend RadioButtons sind.
        initialisiereRadioButtons();

        final JPanel panel = new JPanel();

        // Das neue Layout fuer die Label und Buttons erzeugen.
        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        // Die Ueberschrift fuer die Spalte mit den aktuellen Stufen der
        // Talente setzen.
        panel.add( aktuellUeberschrift, cc.xy( 4, 2 ) );

        // Die Ueberschrift fuer die Spalten mit den RadioButtons setzen.
        for ( int i = 0 ; i < stufenboni.size() ; ++i ) {

            final String text = String.format( "%1$+d", stufenboni.get( i ) );

            final int x = 6 + ( i * 2 );

            panel.add( Util.erzeugeUeberschrift( text ), cc.xy( x, 2 ) );
        }

        // Die einzelnen Zeilen einfuegen.
        for ( int i = 0 ; i < talente.size() ; ++i ) {

            int x = 2;
            final int y = ( 2 * i ) + 4;

            final Talent talent = talente.get( i );

            // Der Name des Talents
            talenteLabel.get( i ).setText( talent.getName() );
            
            panel.add( talenteLabel.get( i ), cc.xy( x, y ) );

            // Die aktuelle Stufe des Talents
            final JLabel labelAktuell = new JLabel();
            aktuell.add( labelAktuell );
            
            if ( held.hat( talent ) ) {

                final int stufe = held.getStufe( talent );

                labelAktuell.setText( String.valueOf( stufe ) );

            } else {

                labelAktuell.setText( "-" );

            }
            labelAktuell.setHorizontalAlignment( SwingConstants.CENTER );
            x += 2;
            panel.add( labelAktuell, cc.xy( x, y ) );

            // Die Spalten mit den RadioButtons
            for ( int j = 0 ; j < stufenboni.size(); ++j ) {
                x += 2;
                panel.add( radioButtons[ i ][ j ], cc.xy( x, y ) );
            }
        }

        final JScrollPane scrollPane = Util.erzeugeScrollPane(  panel );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    /** Erzeugt ein Layout fuer die Benutzeroberflaeche */
    private FormLayout erzeugeLayout() {

        // Das Layout besteht aus mehreren Zeilen und Spalten
        // Die Zeilen mit den Talenten.
        final int zeilen = talente.size();

        // Die Spalten mit den RadioButtons
        final int spalten = stufenboni.size();

        final StringBuilder zeilenLayout = new StringBuilder();
        // Die oberste Zeile hat einen Abstand von 7dlu zum Rand und die
        // Zeile mit den Uebeschriften der Spalten hat die Preferred size
        // der Label als Hoehe.
        zeilenLayout.append( "7dlu, P," );
        for ( int i = 0 ; i < zeilen ; ++i ) {
            // Die Zeilen haben untereinander einen Abstand von 4dlu
            // Die Zeilen mit den RadioButtons richten sich in der Hoehe nach
            // der Preferred size der Buttons.
            zeilenLayout.append( " 4dlu, P," );
        }
        // Der untere Rand ist 7dlu breit.
        zeilenLayout.append( " 7dlu" );

        final StringBuilder spaltenLayout = new StringBuilder();
        // Der linke Rand ist 7dlu breit.
        // Der Inhalt der ersten Spalte ist Rechtsbuendig angeordnet, ist
        // mindestens so breit wie die Default size des Layouts und waechst
        // in der Breite wenn zusaetzlicher Platz da ist.
        spaltenLayout.append( "7dlu, R:D:G, 4dlu, P," );
        for ( int i = 0 ; i < spalten ; ++i ) {
            // Die Spalten untereinander haben einen Abstand von 4dlu.
            // Die Spalten richten sich nach der Preferred size der Buttons.
            spaltenLayout.append( " 4dlu, P," );
        }
        // Die letzte Spalte ist leer und waechst in der Breite wenn
        // zusaetzlicher Platz vorhanden ist.
        spaltenLayout.append( " D:G, 7dlu" );

        return new FormLayout( spaltenLayout.toString(),
                               zeilenLayout.toString() );
    }

    /**
     * Liefert die Koordinaten eines Buttons im Feld {@link #radioButtons}.
     *
     * @param button dessen Koordinaten ermittelt werden sollen.
     * @return die Koordinaten des Buttons im Feld {@link #radioButtons}.
     *      Das gelieferte Feld hat eine Laenge von 2.
     *      An Position 0 steht die Zeile des Buttons.
     *      An Position 1 steht die Spalte des Buttons.
     */
    private int[] getKoordinaten( final JRadioButton button ) {

        final String actionCommand = button.getActionCommand();

        final String[] zs = actionCommand.split( "/" );

        return new int[]{ Integer.valueOf( zs[ 0 ] ),
                          Integer.valueOf( zs[ 1 ] ) };
    }

    /**
     * @return Die Schluessel sind die Talente auf die ein Bonus verteilt wurde.
     *         Die Werte sind die Radiobutton-Spalten die angewaehlt wurden. 
     */
    private Map< Talent, Integer > getSelektion() {
        
        final Map< Talent, Integer > selektion = 
            new HashMap< Talent, Integer >();
        
        for ( int spalte = 0; spalte < stufenboni.size(); ++spalte ) {
            
            for ( int zeile = 0; zeile < talente.size(); ++zeile) {
                
                if ( radioButtons[ zeile ][ spalte ].isSelected() ) {
                    
                    selektion.put( talente.get( zeile ), spalte );
                }
            }
        }
        
        return selektion;
    }
    
    /**
     * @return ausgewaehlten Talentwerte.
     */
    @Override
    public final List< TalentWert > getTalentBoni() {

        final List< TalentWert > talentBoni;
        talentBoni = new ArrayList< TalentWert >( stufenboni.size() );

        // In jeder Spalte gibt es einen Stufenbonus.
        for ( int s = 0, n = stufenboni.size() ; s < n ; ++s ) {

            // Es muss nur das Talent (Zeile) ermittelt werden das gewaehlt
            // ist.
            for ( int z = 0 ; z < radioButtons.length ; ++z )  {

                if ( radioButtons[ z ][ s ].isSelected() ) {

                    // Das gewaehlte Talent in dieser Zeile ist gefunden.
                    final TalentWert wert;
                    wert = new TalentWert( talente.get( z ),
                                           stufenboni.get( s ) );

                    talentBoni.add( wert );

                    break;
                }
            }
        }

        return talentBoni;
    }

    /**
     * Erzeugt gegebenenfalls die benoetigten RadioButtons und sorgt dafuer
     * dass die Buttons auf der Diagonale von links oben nach recht unten
     * angewaehlt sind.
     */
    private void initialisiereRadioButtons() {

        final int zeilen = talente.size();

        final int spalten = stufenboni.size();

        // Sind genuegend Button vorhanden?
        if ( radioButtons.length < zeilen
          || radioButtons[ 0 ].length < spalten ) {

            // Wir haben nicht genuegend Buttons.
            // Es muessen neue erzeugt werden.

            // fuer jede Spalte eine ButtonGroup.
            final ButtonGroup[] buttonGroups;
            buttonGroups = new ButtonGroup[ spalten ];

            for ( int i = 0 ; i < buttonGroups.length ; ++i ) {

                buttonGroups[ i ] = new ButtonGroup();
            }

            radioButtons = new JRadioButton[ zeilen ][ spalten ];

            for ( int z = 0 ; z < zeilen ; ++z ) {
                
                for( int s = 0; s < spalten ; ++s ) {

                    final JRadioButton button = new JRadioButton();

                    button.setHorizontalAlignment( SwingConstants.CENTER );

                    button.addItemListener( this );

                    // Jeder Button kennt seine Position im Feld radioButtons
                    button.setActionCommand( z + "/" + s );

                    radioButtons[ z ][ s ] = button;

                    buttonGroups[ s ].add( button );
                }
            }
            
            for ( int i = talenteLabel.size(); i < zeilen ; ++i ) {
                
                talenteLabel.add( new JLabel() );
            }
        }

        // In jeder Spalte einen Button anwaehlen.
        // Es werden die Button auf der Diagonale von links oben nach rechts
        // unten ausgewaehlt.
        for ( int i = 0 ; i < spalten ; ++i ) {

            radioButtons[ i ][ i ].setSelected( true );
        }
    }

    /**
     * Korrigiert die angewaehlten Button wenn durch eine Aktion des Benutzers
     * in einer Zeile zwei Buttons angewaehlt sind.
     * Die Auswahl des Buttons der in der Zeile schon zuvor angewaehlt war wird
     * versetzt. Angenommen der Benutzer waehlt in Spalte 1 die erste Zeile,
     * zuvor war in Spalte 1 die dritte Zeile angewaehlt.
     * In Zeile eins ist aber schon die 2te Spalte angewaehlt, dann wird in
     * Spalte 2 die dritte Zeile angwaehlt. Die Zeilen werden praktisch einfach
     * vertauscht.
     */
    public final void itemStateChanged(final ItemEvent e) {

        final JRadioButton button = (JRadioButton) e.getSource();

        if ( ItemEvent.SELECTED == e.getStateChange() ) {
            // In einer Spalte wurde ein Button angewaehlt.
            // Evtl ist das in eine Zeile geschehen in der schon ein anderer
            // Button angwaehlt ist. Dann muessen wird das korrigieren.

            // Haben wir gerade eine Korrektur durchgefuehrt?
            if ( korrektur ) {

                korrektur = false;
                // Dann gibt es nichts zu tun.
                return;
            }

            if ( abgewaehlt != null ) {

                // Zeile und Spalte des Buttons der gerade gewaehlt wurde
                // ermitteln.
                final int[] zs = getKoordinaten( button );
                final int zeile = zs[ 0 ];
                final int spalte = zs[ 1 ];

                // Es wird jetzt durch die Zeile gelaufen und geprueft ob ein
                // anderer Button angewaehlt ist.
                for ( int i = 0; i < stufenboni.size() ; ++i ) {

                    // Der neu angewaehlte Button wird uebergangen.
                    if ( spalte != i ) {

                        final JRadioButton buttonInZeile;
                        buttonInZeile = radioButtons[ zeile ][ i ];

                        if ( buttonInZeile.isSelected() ) {
                            // Wir haben in Spalte i einen angwaehlten Button
                            // gefunden.

                            korrektur = true;

                            // In diese Zeile versetzen wir die Auswahl in
                            // Spalte i.
                            final int z = abgewaehlt[ 0 ];

                            radioButtons[ z ][ i ].setSelected( true );

                            break;
                        }
                    }
                }
            }

        } else {

            // Der Benutzer hat in einer Spalte einen Button angwaehlt. Der
            // zuvor in dieser Spalte angewaehlte Button wird zuvor abgewaehlt
            // und wir werden als ItemListener darueber informiert.
            // Wir merken uns die Koordinaten des Buttons der abgewaehlt wurde.
            abgewaehlt = getKoordinaten( button );

        }
    }
    
    public void neuUebersetzen() {
        
        aktuellUeberschrift.setText( 
                I18n.i18n( "PanelTalentBonusFest.aktuell" ) );
        
        final Map< Talent, Integer > selektion = getSelektion();
        
        Collections.sort( talente, Talent.NAME_COMPARATOR );
        
        for ( int i = 0, n = talente.size() ; i < n ; ++i ) {
            
            final Talent talent = talente.get( i );
            
            talenteLabel.get( i ).setText( talent.getName() );
            
            if ( held.hat( talent ) ) {
                
                final int stufe = held.getStufe( talent );
                
                aktuell.get( i ).setText( String.valueOf( stufe ) );
            
            } else {
                
                aktuell.get( i ).setText( "-" );
            }
            
            if ( selektion.containsKey( talent ) ) {
                
                final int spalte = selektion.get( talent );
                
                radioButtons[ i ][ spalte ].setSelected( true );
            }
        }
    }

    /**
     * Bestimmt den Bonus fuer den man mit dem Panel eine Auwahl treffen soll.
     *
     * @param derHeld fuer der Bonus sein soll.
     * @param bonus aus dem es zu waehlen gilt.
     */
    public final void setBonus(
            final Held derHeld,
            final TalentbonusFest bonus ) {

        held = derHeld;
        
        talente = bonus.getTalente( held );

        stufenboni = bonus.getBoni();

        aufbauenBenutzeroberflaeche();

        validate();
        repaint();
    }

}
