/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.gui.action.ActionLadenKultur;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernKultur;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.LookAndFeelAktualisierbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class PanelMetaKultur
extends WechselbaresPanel 
implements Uebersetzbar, LookAndFeelAktualisierbar {

    private final ActionLadenKultur actionLadenKultur;
    
    private final ActionSpeichernKultur actionSpeichernKultur;
    
    private final PanelMetaKulturDiverses diverses;
    
    private final PanelMetaListe< VorNachteil, VorNachteil > empfohlene;

    private final PanelMetaLeittalente leittalente;

    private final PanelMetaModifikationen modifikationen;

    private final PanelMetaVorNachteile nachteile;
    
    private final PanelMetaMoeglicheProfessionen professionen;

    private final PanelMetaSonderfertigkeiten sonderfertigkeiten;
    
    private final PanelMetaTalentboni talentboni;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    private final PanelMetaUngeeigneteVorNachteile ungeeignete;
    
    private final PanelMetaVerbilligteSonderfertigkeiten verbilligteSf;

    private final PanelMetaVerbilligteVarianten verbilligteVarianten;

    private final PanelMetaVorNachteile vorteile;

    private final PanelMetaZauberboni zauberboni;

    private final PanelMetaListe< ElementMitName, Sprache > zweitsprache;

    @SuppressWarnings( "unchecked")
    public PanelMetaKultur(
            final JFrame hauptfenster,
            final Hauptpanel hauptpanel ) {

        uebersetzer = new Uebersetzer();

        actionLadenKultur = new ActionLadenKultur( hauptpanel );
        
        actionSpeichernKultur = new ActionSpeichernKultur( hauptpanel );
        
        diverses = new PanelMetaKulturDiverses( this );
        resetter.einfuegen( diverses );
        
        final SimpleInternalFrame diversesFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Diverses", 
                diverses );
        
        empfohlene = new PanelMetaListe< VorNachteil, VorNachteil >(
                VorNachteil.getVorNachteile(), VorNachteil.NAME_COMPARATOR );
        resetter.einfuegen( empfohlene );
        
        final SimpleInternalFrame empfohleneFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.EmpfohleneVorNachteile",
                empfohlene );
        
        final JButton ladenKultur = new JButton( actionLadenKultur );
        
        leittalente = new PanelMetaLeittalente();
        resetter.einfuegen( leittalente );
        
        final SimpleInternalFrame leittalenteFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Leittalente",
                leittalente );
        
        modifikationen = new PanelMetaModifikationen(
                Collections.EMPTY_LIST );
        resetter.einfuegen( modifikationen );
        
        final SimpleInternalFrame modifikationenFrame = 
            uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Modifikationen",
                modifikationen );

        nachteile = new PanelMetaVorNachteile( 
                VorNachteil.getNachteile(),
                "PanelMetaKultur.Nachteile.Reiter" );
        resetter.einfuegen( nachteile );
        
        final SimpleInternalFrame nachteileFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Nachteile", 
                nachteile );
        
        professionen = new PanelMetaMoeglicheProfessionen();
        resetter.einfuegen( professionen );
        
        final SimpleInternalFrame professionenFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Professionen", 
                professionen );

        sonderfertigkeiten = new PanelMetaSonderfertigkeiten();
        resetter.einfuegen( sonderfertigkeiten );
        
        final SimpleInternalFrame sonderfertigkeitenFrame = 
            uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Sonderfertigkeiten",
                sonderfertigkeiten );

        final JButton speichernKultur = new JButton( actionSpeichernKultur );
        
        talentboni = new PanelMetaTalentboni(
                PanelMetaTalentboni.Modus.KULTUR );
        resetter.einfuegen( talentboni );
        
        final SimpleInternalFrame talentboniFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Talente", 
                talentboni );
        
        ungeeignete = new PanelMetaUngeeigneteVorNachteile();
        resetter.einfuegen( ungeeignete );
        
        final SimpleInternalFrame ungeeigneteFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.UngeeigneteVorNachteile",
                ungeeignete );

        verbilligteSf = new PanelMetaVerbilligteSonderfertigkeiten();
        resetter.einfuegen( verbilligteSf );
        
        final SimpleInternalFrame verbilligteSfFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.VerbilligteSonderfertigkeiten", 
                verbilligteSf );
        
        verbilligteVarianten = new PanelMetaVerbilligteVarianten();
        resetter.einfuegen( verbilligteVarianten );
        
        final SimpleInternalFrame verbilligteVariantenFrame  = 
            uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.VerbilligteVarianten", 
                verbilligteVarianten );
        
        vorteile = new PanelMetaVorNachteile( 
                VorNachteil.getVorteile(),
                "PanelMetaKultur.Vorteile.Reiter" );
        resetter.einfuegen( vorteile );
        
        final SimpleInternalFrame vorteileFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Vorteile", 
                vorteile );

        zauberboni = new PanelMetaZauberboni();
        resetter.einfuegen( zauberboni );
        
        final SimpleInternalFrame zauberboniFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Zauber", 
                zauberboni );
        
        final List< Sprache > sprachen =
            new ArrayList< Sprache >( Talentgruppe.SPRACHEN.size() );

        for ( final Talent sprache : Talentgruppe.SPRACHEN ) {
            sprachen.add( (Sprache) sprache );
        }

        zweitsprache = new PanelMetaListe< ElementMitName, Sprache >( 
                sprachen, 
                Sprache.NAME_COMPARATOR );
        resetter.einfuegen( zweitsprache );
        
        final SimpleInternalFrame zweitspracheFrame = uebersetzer.erzeugeFrame( 
                "PanelMetaKultur.Zweitsprache",
                Util.erzeugeScrollPane( zweitsprache ) );
        
        // Button Bar
        // **********
        final JButton neu = uebersetzer.erzeugeButton( "PanelMetaKultur.Neu" );
        neu.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                eingabenLoeschen();
            }
        });

        final JPanel buttonBar = new JPanel(
                new FormLayout( "4dlu, P, 4dlu", "4dlu, P, 4dlu" ) );

        final CellConstraints cc = new CellConstraints();

        buttonBar.add( ButtonBarFactory.buildLeftAlignedBar(
                    ladenKultur,
                    speichernKultur,
                    neu ),
                cc.xy( 2, 2 ) );

        // Tabbed Pane
        // ***********
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaKultur.Diverses.Reiter", 
                diversesFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaKultur.Modifikationen.Reiter", 
                modifikationenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane,
                "PanelMetaKultur.Professionen.Reiter", 
                professionenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaKultur.Zweitsprache.Reiter", 
                zweitspracheFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaKultur.Talente.Reiter", 
                talentboniFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaKultur.Leittalente.Reiter", 
                leittalenteFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaKultur.Vorteile.Reiter", 
                vorteileFrame );

        uebersetzer.erzeugeTab(
                tabbedPane,
                "PanelMetaKultur.Nachteile.Reiter", 
                nachteileFrame );

        uebersetzer.erzeugeTab(
                tabbedPane,
                "PanelMetaKultur.EmpfohleneVorNachteile.Reiter", 
                empfohleneFrame );

        uebersetzer.erzeugeTab(
                tabbedPane,
                "PanelMetaKultur.UngeeigneteVorNachteile.Reiter", 
                ungeeigneteFrame );

        uebersetzer.erzeugeTab(
                tabbedPane,
                "PanelMetaKultur.Sonderfertigkeiten.Reiter", 
                sonderfertigkeitenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane,
                "PanelMetaKultur.VerbilligteSonderfertigkeiten.Reiter", 
                verbilligteSfFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaKultur.VerbilligteVarianten.Reiter", 
                verbilligteVariantenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane, 
                "PanelMetaKultur.Zauber.Reiter", 
                zauberboniFrame );

        setLayout( new BorderLayout() );

        add( buttonBar, BorderLayout.PAGE_START  );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public void aktualisiereLookAndFeel() {
        
        Util.aktualisiereLookAndFeel(
                actionLadenKultur,
                actionSpeichernKultur );
    }
    
    public final void anzeigen( final Kultur kultur ) {

        diverses.anzeigen( kultur );

        empfohlene.setGewaehlteElemente( kultur.getEmpfohleneVorNachteile() );

        professionen.anzeigen( kultur );

        leittalente.anzeigen( kultur );

        modifikationen.anzeigen( kultur );

        nachteile.anzeigen( kultur.getNachteile() );

        sonderfertigkeiten.anzeigen( kultur.getSonderfertigkeiten() );

        talentboni.anzeigen( kultur );

        ungeeignete.anzeigen( kultur.getUngeeigneteVorNachteile() );

        verbilligteSf.anzeigen( kultur );

        verbilligteVarianten.anzeigen( kultur );

        vorteile.anzeigen( kultur.getVorteile() );

        zauberboni.anzeigen( kultur );

        zweitsprache.setGewaehlteElemente( kultur.getZweitsprachen() );
    }

    /** @return die fuer die Kultur gewaehlte Id. */
    public String getKulturId() {
        return diverses.getKulturId();
    }

    /** @return den Namen, den der Benutzer fuer die Rasse angegeben hat. */
    public String getKulturName() {
        return diverses.getKulturName();
    }

    public boolean kannGewechseltWerden() {
        
        return true;
    }

    /**
     * Setzt das Panel wieder zurueck, damit man eine neue Kultur
     * eingeben kann.
     */
    private void eingabenLoeschen() {

        diverses.eingabenLoeschen();

        empfohlene.eingabenLoeschen();

        professionen.eingabenLoeschen();

        leittalente.eingabenLoeschen();

        modifikationen.eingabenLoeschen();

        nachteile.eingabenLoeschen();

        sonderfertigkeiten.eingabenLoeschen();

        talentboni.eingabenLoeschen();

        ungeeignete.eingabenLoeschen();

        verbilligteSf.eingabenLoeschen();

        verbilligteVarianten.eingabenLoeschen();

        vorteile.eingabenLoeschen();

        zauberboni.eingabenLoeschen();

        zweitsprache.eingabenLoeschen();
    }
    
    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }

    public final void uebertrageDaten( final Kultur kultur ) {

        diverses.uebertrageDaten( kultur );

        kultur.setEmpfohlene( empfohlene.getGewaehlteElemente() );

        kultur.setProfessionen( professionen.getMoeglicheProfessionen() );

        leittalente.uebertrageDaten( kultur );

        modifikationen.uebertrageDaten( kultur );

        kultur.setNachteile( nachteile.getBoni() );

        kultur.setSonderfertigkeiten( sonderfertigkeiten.getBoni() );

        kultur.setTalente( talentboni.getBoni() );

        kultur.setUngeeignete( ungeeignete.getGewaehlteVorNachteile() );

        kultur.setVerbilligte(
                verbilligteSf.getVerbilligteSonderfertigkeiten() );

        kultur.setVerbilligteVariante(
                verbilligteVarianten.getVerbilligteVarianten() );

        kultur.setVorteile( vorteile.getBoni() );

        kultur.setZauberboni( zauberboni.getBoni() );

        kultur.setZweitsprachen( zweitsprache.getGewaehlteElemente() );
    }
}