/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaListe< S, T extends S >
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Fuer jedes Elemente gibt es eine Checkbox. Die Checkbox an position i
     * in der Liste steht fuer das Element an Position i in {@link #elemente}.
     */
    private final List< JCheckBox > checkBoxen;

    /**
     * Der Comparator wird verwendet, um die Elemente in der Liste zu sortieren.
     */
    private final Comparator< S > comparator;
    
    /**
     * Die Liste der waehlbaren Elemente.
     */
    final List< T > elemente;

    /**
     * Das Panel auf dem die Checkboxen angezeigt werden. 
     */
    private final JPanel inhalt;
    
    
    
    /**
     * @param dieElemente aus denen der Benutzer waehlen darf.
     * @param derComparator zum Sortieren der Elemente.
     */
    PanelMetaListe( 
            final List< T > dieElemente,  
            final Comparator< S > derComparator ) {

        
        
        checkBoxen = new ArrayList< JCheckBox >();

        comparator = derComparator;
        
        elemente = new ArrayList< T >( dieElemente );

        Collections.sort( elemente, comparator );
        
        inhalt = new JPanel();
        
        final JScrollPane scrollPane = Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
        
        aufbauenBenutzeroberflaeche();
    }

    protected void aufbauenBenutzeroberflaeche() {

        inhalt.removeAll();

        initCheckBoxen();

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( new FormLayout( "P" ), inhalt );

        builder.setDefaultDialogBorder();

        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            builder.append( checkBoxen.get( i ) );
        }
    }

    /**
     * Liefert die vom Benutzer gewaehlten Elemente.
     * @return
     */
    final List< T > getGewaehlteElemente() {

        final List< T > gewaehlte = new ArrayList< T >();

        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            if ( checkBoxen.get( i ).isSelected() ) {

                gewaehlte.add( elemente.get( i ) );
            }
        }

        return gewaehlte;
    }

    /**
     * Erzeugt die benotigte Anzahl an Checkboxen und setzt ihren Text.
     */
    private void initCheckBoxen() {

        for ( int i = checkBoxen.size(), n = elemente.size(); i < n ; ++i ) {

            checkBoxen.add( new JCheckBox() );
        }

        for ( int i = 0, n = elemente.size(); i < n ; ++i ) {

            checkBoxen.get( i ).setSelected( false );
            checkBoxen.get( i ).setText( elemente.get( i ).toString() );
        }

    }
    
    public void neuUebersetzen() {

        final Collection< T > gewaehlte = getGewaehlteElemente();
        
        Collections.sort( elemente, comparator );
        
        initCheckBoxen();
        
        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {
            
            boolean gewaehlt = gewaehlte.contains(  elemente.get( i ) );
            
            checkBoxen.get( i ).setSelected( gewaehlt );
        }
    }

    /** Waehlt alle Eintraege ab. */
    @SuppressWarnings( "unchecked" )
    public void eingabenLoeschen() {
        setGewaehlteElemente( Collections.EMPTY_LIST );
    }
    
    /**
     * Waehlt die uebergebenen Elemente aus, wenn sie angezeigt werden.
     * @param gewaehlte
     */
    final void setGewaehlteElemente( final Collection< T > gewaehlte ) {

        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            final JCheckBox checkBox = checkBoxen.get( i );

            final boolean gewaehlt = gewaehlte.contains( elemente.get( i ) );

            checkBox.setSelected( gewaehlt );
        }
    }
}
