/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitBonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitBonusGelaendekunde;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitBonusStandard;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitWertBonus;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;
import de.bernhardjung.dsaprogramm.util.Beobachtbar;
import de.bernhardjung.dsaprogramm.util.Beobachter;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

public class PanelMetaSonderfertigkeiten
extends ResetbaresPanel
implements Beobachter, ListSelectionListener, Uebersetzbar {

    /**
     * Der Text neben dem Textfeld in das eine Beschreibung eingegeben werden
     * kann.
     */
    private static final String BESCHREIBUNG =
        "PanelMetaSonderfertigkeiten.Beschreibung";

    /** Die Aufschrift des Knopfes, zum loeschen eines Bonus. */
    private static final String ENTFERNEN =
        "PanelMetaSonderfertigkeiten.Entfernen";

    /**
     * Der Reitertext fuer das Tab, auf dem die ausgewaehlten Vor-/Nachteile
     * angezeigt werden.
     */
    private static final String GEWAEHLTE =
        "PanelMetaSonderfertigkeiten.Gewaehlte";

    /**
     * Der Text auf dem Knopf, mit dem ein neuer Bonus, in die Liste der Boni
     * uebernommen wird.
     */
    private static final String UEBERNEHMEN =
        "PanelMetaSonderfertigkeiten.Uebernehmen";

    /**
     * Der Text fuer die Checkbox, mit der angegeben wird, ob der Benutzer die
     * Variante des gewaehlten Wertes aendern darf.
     */
    private static final String VARIANTE_AENDERBAR_TEXT =
        "PanelMetaSonderfertigkeiten.VarianteAenderbar.Text";

    /**
     * Der Hinweistext fuer die Checkbox.
     */
    private static final String VARIANTE_AENDERBAR_TOOLTIP =
        "PanelMetaSonderfertigkeiten.VarianteAenderbar.Tooltip";

    /**
     * Das Textfeld in das eine Beschreibung fuer den Bonus eingegeben werden
     * kann.
     */
    private final JTextField beschreibung;

    /**
     * Eine Liste aller bisher gewaehlten Vor-/Nachteil-Boni.
     */
    private final JList boniListe;

    /** Das ListModel fuer die Liste {@link #boniListe}. */
    private final 
    CharGenListModel< SonderfertigkeitBonus, SonderfertigkeitBonus > boniModel;

    /**
     * Zeigt den in der Liste ({@link #boniModel}) gewaehlten Bonus an.
     */
    private final JTextArea bonusAnzeige;

    /**
     * Die Werte, die der Benutzer fuer einen neuen Bonus gewaehlt hat.
     */
    private final BeobachtbareListe< SonderfertigkeitWert > liste;

    
    
    private final JButton uebernehmen;

    /** Uebersetzt die Texte der Benutzeroberlflaeche. */
    private final Uebersetzer uebersetzer;
    
    /**
     * Mit dieser Checkbox kann angegeben werden ob der Benutzer die Variante
     * der Werte bei der Erschaffung frei waehlen kann.
     */
    private final JCheckBox varianteAenderbar;

    PanelMetaSonderfertigkeiten() {

        uebersetzer = new Uebersetzer();
    
        
        
        beschreibung = new JTextField();

        boniModel = 
            new CharGenListModel< SonderfertigkeitBonus, SonderfertigkeitBonus >();

        boniListe = uebersetzer.erzeugeListe( boniModel );
        boniListe.addListSelectionListener( this );
        boniListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

        bonusAnzeige = new JTextArea();
        bonusAnzeige.setLineWrap( true );
        bonusAnzeige.setWrapStyleWord( true );
        bonusAnzeige.setEditable( false );

        liste = new BeobachtbareListe< SonderfertigkeitWert >();
        liste.anmelden( this );

        uebernehmen = uebersetzer.erzeugeButton( UEBERNEHMEN );
        uebernehmen.setEnabled( false );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        varianteAenderbar = 
            uebersetzer.erzeugeCheckBox( VARIANTE_AENDERBAR_TEXT );
        varianteAenderbar.setToolTipText( VARIANTE_AENDERBAR_TOOLTIP );
        varianteAenderbar.setEnabled( false );

        aufbauenBenutzeroberflaeche();
    }

    public final void anzeigen( final Collection< SonderfertigkeitBonus > boni ) {

        boniListe.clearSelection();

        boniModel.setElements( boni );
    }

    private void aufbauenBenutzeroberflaeche() {

        final JButton entfernen = uebersetzer.erzeugeButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                entfernen( e );
            }
        });

        final JButton gelaendekunde = uebersetzer.erzeugeButton( 
                SonderfertigkeitBonusGelaendekunde.BESCHREIBUNG );
        gelaendekunde.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                boniModel.addElement( SonderfertigkeitBonus.GELAENDEKUNDE );
            }
        });

        final PanelMetaSonderfertigkeitenGewaehlte gewaehlte =
            new PanelMetaSonderfertigkeitenGewaehlte( liste );
        resetter.einfuegen( gewaehlte );
        
        final PanelMetaSonderfertigkeitenWaehlbar allgemein =
            erzeugePanel( Sonderfertigkeitgruppe.ALLGEMEIN );
        resetter.einfuegen( allgemein );

        final PanelMetaSonderfertigkeitenWaehlbar kampf =
            erzeugePanel( Sonderfertigkeitgruppe.KAMPF );
        resetter.einfuegen( kampf );

        final PanelMetaSonderfertigkeitenWaehlbar magische =
            erzeugePanel( Sonderfertigkeitgruppe.MAGISCHE );
        resetter.einfuegen( magische );

        final PanelMetaSonderfertigkeitenWaehlbar talent =
            erzeugePanel( Sonderfertigkeitgruppe.TALENTSPEZIALISIERUNG );
        resetter.einfuegen( talent );

        final PanelMetaSonderfertigkeitenWaehlbar waffen =
            erzeugePanel( Sonderfertigkeitgruppe.WAFFENSPEZIALISIERUNG );
        resetter.einfuegen( waffen );

        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                GEWAEHLTE, 
                gewaehlte );
        
        uebersetzer.erzeugeTab( 
                tabbedPane,
                Sonderfertigkeitgruppe.ALLGEMEIN.getLokalisierungsschluesselKuerzel(),
                allgemein );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                Sonderfertigkeitgruppe.KAMPF.getLokalisierungsschluesselKuerzel(),
                kampf );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                Sonderfertigkeitgruppe.MAGISCHE.getLokalisierungsschluesselKuerzel(),
                magische );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                Sonderfertigkeitgruppe.TALENTSPEZIALISIERUNG.getLokalisierungsschluesselKuerzel(),
                talent );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                Sonderfertigkeitgruppe.WAFFENSPEZIALISIERUNG.getLokalisierungsschluesselKuerzel(),
                waffen );

        final JPanel textfeld =
            new JPanel( new FormLayout( "P, 4dlu, F:P:G", "P" ) );

        final CellConstraints cc = new CellConstraints();

        textfeld.add( uebersetzer.erzeugeLabel( BESCHREIBUNG ), cc.xy( 1, 1 ) );

        textfeld.add( beschreibung, cc.xy( 3, 1 ) );

        setLayout( new FormLayout( "7dlu, F:75px:G(0.2), 4dlu, F:100px:G(0.8), 7dlu",
            "7dlu, F:50px:G(0.2), 4dlu, F:100px:G(0.8), " +
            "4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 7dlu" ) );

        add( new JScrollPane( bonusAnzeige ), cc.xy( 2, 2 ) );

        add( new JScrollPane( boniListe ), cc.xywh( 2, 4, 1, 7 ) );

        add( entfernen, cc.xy( 2, 12 ) );

        add( tabbedPane, cc.xywh( 4, 2, 1, 3 ) );

        add( textfeld, cc.xy( 4, 6 ) );

        add( varianteAenderbar, cc.xy( 4, 8 ) );

        add( uebernehmen, cc.xy( 4, 10 ) );

        add( gelaendekunde, cc.xy( 4, 12 ) );
    }

    /**
     * Wird aufgerufen, wenn der gewaehlte Bonus aus der Liste
     * {@link #boniModel} entfernt werden soll.
     */
    private void entfernen( final ActionEvent e ) {

        final int index = boniListe.getSelectedIndex();

        if ( index >= 0 ) {

            boniModel.removeElementAt( index );
        }
    }

    private PanelMetaSonderfertigkeitenWaehlbar erzeugePanel(
            final List< Sonderfertigkeit > sonderfertigkeiten ) {

        final PanelMetaSonderfertigkeitenWaehlbar panel =
            new PanelMetaSonderfertigkeitenWaehlbar( liste );

        panel.setWerte( getWerte( sonderfertigkeiten ) );

        return panel;
    }

    /**
     * @return eine Liste der Vor-/Nachteil-Boni.
     */
    public List< SonderfertigkeitBonus > getBoni() {
        return boniModel.getElements();
    }

    public Collection< SonderfertigkeitWert > getWerte(
            final Collection< Sonderfertigkeit > sonderfertigkeiten ) {

        final Collection< SonderfertigkeitWert > werte =
            new ArrayList< SonderfertigkeitWert >( sonderfertigkeiten.size() );

        for ( final Sonderfertigkeit sonderfertigkeit : sonderfertigkeiten ) {

            final SonderfertigkeitWert wert;

            if ( sonderfertigkeit.hatVarianten() ) {

                final List< Variante > varianten = sonderfertigkeit.getVarianten( null );

                final Variante variante;

                if ( varianten.size() > 0 ) {

                    variante = varianten.get( 0 );

                } else {

                    variante = new Variante();
                }

                wert = new SonderfertigkeitWert( sonderfertigkeit, variante );

            } else {

                wert = new SonderfertigkeitWert( sonderfertigkeit );
            }

            werte.add( wert );
        }

        return werte;
    }

    public void melden( final Beobachtbar b, final Object arg ) {

        uebernehmen.setEnabled( liste.size() > 0 );

        boolean hatVarianten = false;

        for ( final SonderfertigkeitWert wert : liste.getElemente() ) {

            if ( wert.getSonderfertigkeit().hatVarianten() ) {

                hatVarianten = true;

                break;
            }
        }

        if ( hatVarianten ) {

            varianteAenderbar.setEnabled( true );

        } else {

            varianteAenderbar.setEnabled( false );
            varianteAenderbar.setSelected( false );
        }
    }
    
    /**
     * Uebersetzt das Panel neu.
     */
    public void neuUebersetzen() {
         
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Loescht alle gewaehlten Boni aus der Liste und setzt die Panel zurueck.
     */
    public void eingabenLoeschen() {
        boniModel.clear();
        liste.clear();
    }

    private void uebernehmen() {

        final List< SonderfertigkeitWertBonus > wertBoni =
            new ArrayList< SonderfertigkeitWertBonus >();

        for ( final SonderfertigkeitWert wert : liste.getElemente() ) {

            if ( wert.getSonderfertigkeit().hatVarianten() ) {

                final String id = wert.getVariante().getId();

                wertBoni.add( new SonderfertigkeitWertBonus( wert, id ) );

            } else {

                wertBoni.add( new SonderfertigkeitWertBonus( wert, null ) );

            }
        }

        String beschreibungString = beschreibung.getText();

        if ( beschreibungString.trim().equals( "" ) ) {

            beschreibungString = null;

        }

        final SonderfertigkeitBonus bonus =
            new SonderfertigkeitBonusStandard(
                    beschreibungString,
                    varianteAenderbar.isSelected(),
                    wertBoni );

        boniModel.addElement( bonus );

        liste.clear();
        beschreibung.setText( "" );
    }

    /**
     * Wird aufgerufen wenn in der Liste {@link #boniModel} ein Talentbonus
     * gewaehlt wird.
     *
     * @param e
     */
    public void valueChanged( final ListSelectionEvent e ) {

        if ( e.getValueIsAdjusting() ) {

            return;
        }

        final Object gewaehlt = boniListe.getSelectedValue();

        if ( null != gewaehlt ) {

            bonusAnzeige.setText( gewaehlt.toString() );

        } else {

            bonusAnzeige.setText( "" );
        }
    }

}
