/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.util.Wuerfeltabelle;
import de.bernhardjung.dsaprogramm.bausteine.util.Wuerfeltabelle.Tabelleneintrag;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaWuerfeltabelle
extends ResetbaresPanel
implements ChangeListener, Uebersetzbar {

    /**
     * Die Combobox erlaubt es das Ergebnis fuer ein Intervall zu waehlen,
     * also z.B. bei "7 -9: Augenfarbe rot" das rot.
     */
    private final List< JComboBox > comboBoxen;

    private final ElementQuelle elementQuelle;

    /**
     * Das Label zeigt den Anfang eines Intervalls an auf das gewurfelt
     * werden muss, z.B. beim Eintrag "7 - 9: Augenfarbe rot" die 7. 
     */
    private final List< JLabel > label;

    
    
    /**
     * Der Spinner zeigt das Ende des Intervalls an, also z.B. bei 
     * "7 - 9: Augenfarbe rot" die 9.
     */
    private final List< JSpinner > spinner;

    PanelMetaWuerfeltabelle( final ElementQuelle dieElementQuelle ) {

        
        
        comboBoxen = new ArrayList< JComboBox >( 20 );

        elementQuelle = dieElementQuelle;

        label = new ArrayList< JLabel >( 20 );

        spinner = new ArrayList< JSpinner >( 20 );

        for ( int i = 0 ; i < 20 ; ++i ) {

            comboBoxen.add( erzeugeComboBox( elementQuelle.getElemente() ) );

            String text;

            if ( 0 == i ) {

                text = "1";

            } else {

                text = "21";
            }

            label.add( new JLabel( text ) );

            spinner.add( erzeugeSpinner( i ) );
        }

        final FormLayout layout = new FormLayout( "P, 4dlu, P, 4dlu, P, 4dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.setDefaultDialogBorder();

        for ( int i = 0 ; i < 20 ; ++i ) {

            builder.append( label.get( i ) );

            builder.append( new JLabel( "-" ) );

            builder.append( spinner.get( i ) );

            builder.append( comboBoxen.get( i ) );
        }
        
        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Wuerfeltabelle tabelle ) {

        final List< Tabelleneintrag > eintraege =
            tabelle.getTabelleneintraege();

        for ( int i = 0, n = eintraege.size() ; i < n ; ++i ) {

            final Tabelleneintrag eintrag = eintraege.get( i );

            spinner.get( i ).setValue( eintrag.bis );

            final Object ergebnis = elementQuelle.getElement( eintrag.id );
            
            comboBoxen.get( i ).setSelectedItem( ergebnis );
        }
    }

    private JComboBox erzeugeComboBox( final Vector< Object > elemente ) {

        final JComboBox comboBox = new JComboBox( elemente );

        comboBox.setEditable( true );

        return comboBox;
    }

    private JSpinner erzeugeSpinner( final int index ) {

        final JSpinner spinner = Util.erzeugeSpinner( 20, index+1, 20, 1 );

        spinner.setName( String.valueOf( index ) );

        spinner.addChangeListener( this );

        return spinner;
    }

    public Wuerfeltabelle getWuerfeltabelle() {

        final Wuerfeltabelle tabelle = new Wuerfeltabelle();

        int von = 1;
        for ( int i = 0, n = spinner.size()
                ; ( i < n ) && ( von <= 20 ) ; ++i ) {

            final int bis = (Integer) spinner.get( i ).getValue();

            Object eintrag = comboBoxen.get( i ).getSelectedItem();
            
            final String id = elementQuelle.getId( eintrag );

            tabelle.eintragen( von, bis, id );

            von = bis + 1;
        }

        return tabelle;
    }
    
    /** Ubersetzt das Panel neu. */
    public void neuUebersetzen() {
        
        final Wuerfeltabelle tabelle = getWuerfeltabelle();
        
        final Vector< Object > elemente = elementQuelle.getElemente();
        
        for ( final JComboBox comboBox : comboBoxen ) {
            
            comboBox.setModel( new DefaultComboBoxModel( elemente ) );
        }
        
        anzeigen( tabelle );
    }
    
    public void stateChanged( final ChangeEvent e ) {

        final String indexString = ( (JSpinner) e.getSource() ).getName();

        final int index = Integer.valueOf( indexString );

        int i = index;

        if ( index < 19 ) {

            final int wert = (Integer) spinner.get( index ).getValue();

            label.get( index + 1 ).setText( String.valueOf( wert + 1 ) );
        }

        while ( i > 0 ) {

            final int wert = (Integer) spinner.get( i ).getValue();

            final JSpinner spinnerDavor = spinner.get( i - 1 );

            final int wertDavor = (Integer) spinnerDavor.getValue();

            if ( wert <= wertDavor ) {

                spinnerDavor.setValue( wert - 1 );

                label.get( i ).setText( String.valueOf( wert ) );

                --i;

            } else {

                break;

            }
        }

        i = index;

        while ( i < 19 ) {

            final int wert = (Integer) spinner.get( i ).getValue();

            final JSpinner spinnerDanach = spinner.get( i + 1 );

            final int wertDanach = (Integer) spinnerDanach.getValue();

            if ( ( wert >= wertDanach ) && ( wert < 20 ) ) {

                spinnerDanach.setValue( wert + 1 );

                label.get( i + 1 ).setText( String.valueOf( wert + 1 ) );

                ++i;

            } else {

                break;

            }
        }
    }

    public interface ElementQuelle {

        public Object getElement( final String id );
        
        public Vector< Object > getElemente();

        public String getId( final Object element );
    }
}
