/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.BorderLayout;
import java.awt.Font;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.erschaffung.Erschaffung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.PanelHeldendaten;
import de.bernhardjung.dsaprogramm.gui.PanelSonderfertigkeiten;
import de.bernhardjung.dsaprogramm.gui.PanelTalente;
import de.bernhardjung.dsaprogramm.gui.PanelZauber;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernHeldXml;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

public class PanelSteigern 
extends WechselbaresPanel
implements Observer, Uebersetzbar {

    /**
     * Die Frage die dem Benutzer gestellt wird, wenn er das Steigern des
     * aktuellen Helden abbrechen will, es aber noch ungespeicherte Aenderungen
     * gibt.
     */
    private static final String ABBRECHEN_FRAGE = 
        "PanelSteigern.SteigernAbbrechen.Frage";
    
    /** Der Reiter-Text auf der Registerkarte auf der die AP angezeigt werden.*/
    private static final String AP = "PanelSteigern.Ap";

    /** Der Text fuer den InternalFrame in dem die AP angezeigt werden. */
    private static final String ABENTEUERPUNKTE =
        "PanelSteigern.Abenteuerpunkte";

    private static final String AP_ZUR_VERFUEGUNG = 
        "PanelSteigern.ApZurVerfuegung";

    private static final String AUSRUESTUNG = "PanelSteigern.Ausruestung";

    private static final String BASISWERTE = "PanelSteigern.Basiswerte";

    private static final String EIGENSCHAFTEN = "PanelSteigern.Eigenschaften";

    private static final String HELDENDATEN = "PanelSteigern.Heldendaten";

    private static final String SONDERFERTIGKEITEN = 
        "PanelSteigern.Sonderfertigkeiten";

    private static final String TALENTE = "PanelSteigern.Talente";

    private static final String VOR_NACHTEILE = "PanelSteigern.VorNachteile";

    private static final String ZAUBER = "PanelSteigern.Zauber";

    private final ActionSpeichernHeldXml actionSpeichern;
    
    private final PanelAp ap;

    private final JLabel apGuthaben;

    private final PanelAusruestung ausruestung;

    private final PanelEigenschaftenSteigern eigenschaften;

    private final PanelBasiswerteSteigern basiswerte;

    /** Der Held der aktuell gesteigert wird. */
    private Held held;
    
    /**
     * Wenn der Held im Panel gesetzt wird wird hier ein Kopie von ihm angelegt.
     * Die Kopie wird bei jedem Speichern aktualisiert. Indem man die Kopie
     * mit dem {@link #held}en vergleicht der gesteigert wird kann man sehen,
     * ob es ungespeicherte Änderungen gibt.
     */
    private Held heldKopie;

    private final PanelHeldendaten heldendaten;

    
    
    private final PanelSonderfertigkeiten sonderfertigkeiten;

    private final PanelTalente talente;

    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    private final PanelVorNachteileSteigern vorNachteile;

    private final PanelZauber zauber;

    public PanelSteigern( final ActionSpeichernHeldXml dieActionSpeichern ) {

        uebersetzer = new Uebersetzer();
    
        
        
        actionSpeichern = dieActionSpeichern;
        
        ap = new PanelAp();

        apGuthaben = new JLabel();
        apGuthaben.setHorizontalAlignment( JLabel.RIGHT );
        final Font schrift = getFont();
        final int schriftgroesse = schrift.getSize();
        final float neueSchriftgroesse = schriftgroesse * 1.5F;
        final Font grosseSchrift =
            schrift.deriveFont( Font.BOLD, neueSchriftgroesse );
        apGuthaben.setFont( grosseSchrift );

        ausruestung = new PanelAusruestung();
        resetter.einfuegen( ausruestung );

        basiswerte = new PanelBasiswerteSteigern();
        resetter.einfuegen( basiswerte );

        eigenschaften = new PanelEigenschaftenSteigern();
        resetter.einfuegen( eigenschaften );

        heldendaten = new PanelHeldendaten();
        resetter.einfuegen( heldendaten );

        sonderfertigkeiten = new PanelSonderfertigkeiten( true );
        resetter.einfuegen( sonderfertigkeiten );

        talente = new PanelTalente( true );
        resetter.einfuegen( talente );

        vorNachteile = new PanelVorNachteileSteigern();
        resetter.einfuegen( vorNachteile );

        zauber = new PanelZauber( true );
        resetter.einfuegen( zauber );

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final JPanel apPanel = new JPanel(
                new FormLayout( "7dlu, F:P:G, 7dlu", "7dlu, P, 7dlu ") );
        apPanel.add( apGuthaben, new CellConstraints( 2, 2 ) );

        final JTabbedPane tabbedPane = new JTabbedPane();
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab(
                tabbedPane,
                HELDENDATEN,
                uebersetzer.erzeugeFrame( HELDENDATEN, heldendaten ) );

        uebersetzer.erzeugeTab(
                tabbedPane,
                AP,
                uebersetzer.erzeugeFrame( ABENTEUERPUNKTE, ap ) );

        uebersetzer.erzeugeTab(
                tabbedPane,
                EIGENSCHAFTEN,
                uebersetzer.erzeugeFrame( EIGENSCHAFTEN, eigenschaften ) );

        uebersetzer.erzeugeTab(
                tabbedPane,
                BASISWERTE,
                uebersetzer.erzeugeFrame( BASISWERTE, basiswerte ) );

        uebersetzer.erzeugeTab( tabbedPane, TALENTE, talente );

        uebersetzer.erzeugeTab( tabbedPane, VOR_NACHTEILE, vorNachteile );

        uebersetzer.erzeugeTab( 
                tabbedPane, 
                SONDERFERTIGKEITEN, 
                sonderfertigkeiten );

        uebersetzer.erzeugeTab( tabbedPane, ZAUBER, zauber );

        uebersetzer.erzeugeTab( tabbedPane, AUSRUESTUNG, ausruestung );

        setLayout( new BorderLayout() );

        add( apPanel, BorderLayout.PAGE_START );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public PanelSteigern debugSetHeld( final Hauptpanel hauptpanel ) {
        
        final Erschaffung erschaffung = new Erschaffung( hauptpanel );
        
        erschaffung.debugSchnellerschaffung(
                "Rasse.Mittellaender",
                "Kultur.Bornland",
                "Profession.Magier" );
        
        setHeld( erschaffung.debugGetManager().getHeld(), false );
        
        return this;
    }

    /** @return den Held der aktuell gesteigert wird. */
    public final Held getHeld() {
        return held;
    }
    
    /**
     * Muss aufgerufen werden nachdem der Held gespeichert wurde, damit das
     * Panel weiß, dass es keine ungespeicherten Aenderungen gibt.
     */
    public final void heldGespeichert() {
        
        heldKopie = new Held( held );
    }
    
    public boolean kannGewechseltWerden() {
        
        if ( null == heldKopie || held.hatAndereWerteAls( heldKopie ) ) {
            
            // Fragen, ob die Aenderungen gespeichert werden sollen.
            int antwort = JOptionPane.showConfirmDialog( 
                    this, 
                    I18n.i18n( ABBRECHEN_FRAGE ),
                    null,
                    JOptionPane.YES_NO_CANCEL_OPTION );
            
            if ( JOptionPane.YES_OPTION == antwort ) {
            
                return actionSpeichern.speichern();
                
            } else {
                
                return JOptionPane.NO_OPTION == antwort;
            }
            
        } else {
            
            return true;
        }
    }

    public void neuUebersetzen() {
    
        uebersetzer.neuUebersetzen();
        
        updateApGuthaben();
    }
    
    /**
     * @param derHeld der im Panel gesteigert wird.
     * @param heldIstNeu {@code true}, bedeutet das der Held gerade neu 
     *        erschaffen wurde und noch nicht gespeichert ist. {@code false}
     *        bedeutet dass der Held von der Festplatte geladen wurde.
     */
    public final void setHeld( final Held derHeld, boolean heldIstNeu ) {

        held = derHeld;
        
        if ( heldIstNeu ) {
        
            heldKopie = null;
        
        } else {
            
            heldKopie = new Held( held );
        }

        held.addObserver( this );

        ap.setHeld( held );

        ausruestung.setHeld( held );

        basiswerte.setHeld( held );

        eigenschaften.setHeld( held );

        heldendaten.setHeld( held );

        sonderfertigkeiten.setHeld( held );

        talente.setHeld( held );

        vorNachteile.setHeld( held );

        zauber.setHeld( held );

        update( held, null );
    }

    /**
     * @param o der Held, der vom Panel beobachtet wird.
     * @param arg ungenutzt.
     */
    public final void update( final Observable o, final Object arg ) {

        updateApGuthaben();
    }

    /**
     * Bringt die Anzeige fuer das AP-Guthaben am oberen Rand des Fensters auf
     * den neuesten Stand.
     */
    private void updateApGuthaben()  {

        apGuthaben.setText( String.format( 
                I18n.i18n( AP_ZUR_VERFUEGUNG ),
                held.getApGuthaben() ) );
    }
}
