/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.sonderfertigkeiten;

import java.util.ArrayList;
import java.util.List;

public class SonderfertigkeitWertListe implements Cloneable {

    private List< SonderfertigkeitWert > werte;
    
    public SonderfertigkeitWertListe() {
        
        werte = new ArrayList< SonderfertigkeitWert >();
    }
    
    /**
     * Haengt ein Element an die Liste an.
     * 
     * @param wert der hinten an die Liste angehaengt werden soll.
     */
    public final void add( final SonderfertigkeitWert wert ) {
       
        werte.add( wert );
    }
    
    /**
     * Leert den Inhalt der Liste.
     */
    public final void clear() {
        
        werte.clear();
    }
    
    /**
     * Erzeugt eine Deep Copy der dieser Liste.
     */
    @Override
    public final SonderfertigkeitWertListe clone() {
        
        SonderfertigkeitWertListe clone = null;
        
        try {
            
            clone = (SonderfertigkeitWertListe) super.clone();
        
            clone.werte = SonderfertigkeitWert.deepCopy( werte );
            
        } catch ( CloneNotSupportedException e ) {
            
            e.printStackTrace();
        }
        
        return clone;
    }
    
    @Override
    public final boolean equals( final Object obj ) {
        
        if ( null == obj 
                || ! obj.getClass().equals( getClass() ) ) {
            
            return false;
        }
        
        final SonderfertigkeitWertListe liste = (SonderfertigkeitWertListe) obj;
        
        return werte.equals( liste.werte );
    }
    /**
     * Liefert den Wert von der uebergebenen Position der Liste.
     * 
     * @param index der Position an der das gesucht Element steht.
     * @return Das Element von der gegebenen Position.
     */
    public final SonderfertigkeitWert get( final int index ) {
        
        return werte.get( index );
    }
    
    /**
     * Liefert die Position der Liste an der sich das gesuchte Element befindet.
     * 
     * @param wert dessen Position gesucht wird.
     * @return Der Index der Position des Wertes oder -1 wenn der Wert nicht
     *         in der Liste ist.
     */
    public final int indexOf( final SonderfertigkeitWert wert ) {
        
        for ( int i = 0, n = size() ; i < n ; i++ ) {
            
            if ( get( i ).hatGleicheSfUndVariante( wert ) ) {
                
                return i;
            }
        }
        
        return -1;
    }
    
    /**
     * Entfernt einen Wert aus der Liste. Macht nichts, wenn der Wert nicht
     * in der Liste ist.
     * 
     * @param wert der aus der Liste entfernt werden soll.
     */
    public final void remove( final SonderfertigkeitWert wert ) {
        
        int index = indexOf( wert );
        
        if ( 0 <= index ) {
            
            werte.remove( index );
        }
    }
    /**
     * @return Die Anzahl Elemente in der Liste.
     */
    public final int size() {
        
        return werte.size();
    }
}
