/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;

class VoraussetzungNichtFreiWaehlbar extends Voraussetzung {

    /**
     * @return eine leere Collection
     */
    @Override
    @SuppressWarnings( "unchecked" )
    public Collection< Object > getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /**
     * Prueft ob das Talent vorgegeben ist.
     */
    @Override
    public KonfliktBeschreibung pruefen( final Held held, final Talent talent ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( ! held.hatVorgegeben( talent ) ) {

            beschreibung.einfuegen( 
                    I18n.i18n( "VoraussetzungNichtFreiWaehlbar" ) );
        }

        return beschreibung;
    }

}
