/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.ArrayList;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

class VoraussetzungSteigernSchrift extends Voraussetzung {

    VoraussetzungSteigernSchrift() {
    }
    
    /** @return alle Schriften. */
    public Collection<Object> getAbhaengigkeiten() {
        return new ArrayList< Object >( Talentgruppe.SCHRIFTEN );
    }
    
    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final Talent talent ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        if ( held.hat( talent ) 
                && ( held.getStufe( talent ) > 10 ) ) {
            
            boolean erfuellt = false;
            
            for ( final Talent schrift : Talentgruppe.SCHRIFTEN ) {
                
                if ( held.hat( schrift ) 
                        && ( held.getStufe( schrift ) >= 6 ) ) {
                    
                    erfuellt = true;
                    break;
                }
            }
            
            if ( ! erfuellt ) {
                
                beschreibung.einfuegen( 
                        I18n.i18n( "VoraussetzungSteigernSchrift" ) );
            }
        }

        return beschreibung;
    }

}
