/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.anpasser;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public interface Anpasser {

    /**
     * Dieser Anpasser fuer den Nachteil Albino aendert die Haar und
     * Augenfarbe.
     */
    public static final Anpasser ALBINO = new AnpasserAlbino();

    /**
     * Aktualisiert das Alter des Helden.
     */
    public static final Anpasser ALTER = new AnpasserAlter();

    /**
     * Aendert das Gewicht des Helden.
     */
    public static final Anpasser FETTLEIBIG = new AnpasserFettleibig();

    /**
     * Setzt die GE eines Helden bei Erhalt eines Vor-/Nachteils auf 14.
     */
    public static final Anpasser GE_14 =
        new AnpasserEigenschaft( Eigenschaft.GE, 14 );

    /**
     * Dieser Anpasser fuer den Vorteil Gefahreninstinkt fuegt dem Helden das
     * Talent Gefahreninstinkt hinzu, bzw entfernt es wieder.
     */
    public static final Anpasser GEFAHRENINSTINKT =
        new AnpasserTalent( Talent.GEFAHRENINSTINKT );

    /** Der Anpasser fuer Herausragende Fingerfertigkeit. */
    public static final Anpasser HERAUSRAGENDE_FF =
        new AnpasserHerausragendeEigenschaft( Eigenschaft.FF );

    /** Der Anpasser fuer Herausragende Gewandtheit */
    public static final Anpasser HERAUSRAGENDE_GE =
        new AnpasserHerausragendeEigenschaft( Eigenschaft.GE );

    /** Der Anpasser fuer Herausragende Intuition */
    public static final Anpasser HERAUSRAGENDE_IN =
        new AnpasserHerausragendeEigenschaft( Eigenschaft.IN );

    /** Der Anpasser fuer Herausragende Klugheit */
    public static final Anpasser HERAUSRAGENDE_KL =
        new AnpasserHerausragendeEigenschaft( Eigenschaft.KL );

    /** Der Anpasser fuer Herausragende Konstitution */
    public static final Anpasser HERAUSRAGENDE_KO =
        new AnpasserHerausragendeEigenschaft( Eigenschaft.KO );

    /** Der Anpasser fuer Herausragende Koerperkraft */
    public static final Anpasser HERAUSRAGENDE_KK =
        new AnpasserHerausragendeEigenschaft( Eigenschaft.KK );

    /** Der Anpasser fuer Herausragendes Charisma */
    public static final Anpasser HERAUSRAGENDES_CH =
        new AnpasserHerausragendeEigenschaft( Eigenschaft.CH );

    /** Der Anpasser fuer Herausragender Mut */
    public static final Anpasser HERAUSRAGENDER_MU =
        new AnpasserHerausragendeEigenschaft( Eigenschaft.MU );

    /**
     * Dieser Anpasser fuer den Vorteil Magiegespuer fuegt dem Helden das
     * Talent Magiegespuer hinzu, bzw entfernt es wieder.
     */
    public static final Anpasser MAGIEGESPUER =
        new AnpasserTalent( Talent.MAGIEGESPUER );

    /** Der Anpasser fuer Miserable Fingerfertigkeit. */
    public static final Anpasser MISERABLE_FF =
        new AnpasserMiserableEigenschaft( Eigenschaft.FF );

    /** Der Anpasser fuer Miserable Gewandtheit */
    public static final Anpasser MISERABLE_GE =
        new AnpasserMiserableEigenschaft( Eigenschaft.GE );

    /** Der Anpasser fuer Miserable Intuition */
    public static final Anpasser MISERABLE_IN =
        new AnpasserMiserableEigenschaft( Eigenschaft.IN );

    /** Der Anpasser fuer Miserable Klugheit */
    public static final Anpasser MISERABLE_KL =
        new AnpasserMiserableEigenschaft( Eigenschaft.KL );

    /** Der Anpasser fuer Miserable Konstitution */
    public static final Anpasser MISERABLE_KO =
        new AnpasserMiserableEigenschaft( Eigenschaft.KO );

    /** Der Anpasser fuer Miserable Koerperkraft */
    public static final Anpasser MISERABLE_KK =
        new AnpasserMiserableEigenschaft( Eigenschaft.KK );

    /** Der Anpasser fuer Miserables Charisma */
    public static final Anpasser MISERABLES_CH =
        new AnpasserMiserableEigenschaft( Eigenschaft.CH );

    /** Der Anpasser fuer Miserabler Mut */
    public static final Anpasser MISERABLER_MU =
        new AnpasserMiserableEigenschaft( Eigenschaft.MU );

    /**
     * Dieser Anpasser fuer den Vorteil Prophezeien fuegt dem Helden das
     * Talent Prophezeien hinzu, bzw entfernt es wieder.
     */
    public static final Anpasser PROPHEZEIEN =
        new AnpasserTalent( Talent.PROPHEZEIEN );

    /**
     * Setzt den SO eines Helden bei Erhalt eines Vor-/Nachteils auf 8.
     */
    public static final Anpasser SO_8 =
        new AnpasserEigenschaft( Eigenschaft.SO, 8 );

    /** Anpasser fuer den Vorteil Uebernatuerlichen Begabung. */
    public static final Anpasser UEBERNATUERLICHE_BEGABUNG =
        new AnpasserUebernatuerlicheBegabung();

    /**
     * Dieser Anpasser fuer den Vorteil Zwergennase fuegt dem Helden das
     * Talent Zwergennase hinzu, bzw entfernt es wieder.
     */
    public static final Anpasser ZWERGENNASE =
        new AnpasserTalent( Talent.ZWERGENNASE );

    /**
     * Wird aufgerufen nachdem der Held den Vor-/Nachteil erhalten hat.
     *
     * @param held der den Vorteil erhalten hat.
     * @param wert den der Held erhalten hat.
     */
    public void eingefuegt( final Held held, VorNachteilWert wert );

    /**
     * Wird aufgerufen nachdem der Held den Vor-/Nachteil verloren hat.
     *
     * @param held der den Vorteil verloren hat.
     * @param wert der vom Helden entfernt wurde.
     */
    public void entfernt( final Held held, VorNachteilWert wert );

    /**
     * Wird aufgerufen, wenn sich die Variante eines Wertes geaendert hat.
     *
     * @param held
     *            dem der geaenderte Wert gehoert.
     * @param alteVariante
     *            Die bisherige Variante des Wertes.
     * @param neueVariante
     *            Die neue Variante des Wertes.
     */
    public void geaendertVariante(
            Held held,
            Object alteVariante,
            Object neueVariante );

    /**
     * Wird aufgerufen wen sich die Stufe des Vor-/Nachteils geaendert hat.
     *
     * @param held der den Vor-/Nachteil besitzt.
     * @param wert dessen Stufe sich geaendert hat.
     */
    public void stufeGeaendert( final Held held, final VorNachteilWert wert );
}
