/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.anpasser;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class AnpasserSonderfertigkeit implements Anpasser {

    /**
     * Gibt an ob die Sonderfertigkeit schon vorher vom Benutzer gewaehlt
     * wurde. Wenn dies der Fall ist so soll sie nicht automatisch entfernt
     * werden, wenn der Vor-/Nachteil abgewaehlt wird.
     */
    private boolean vomBenutzerGewaehlt;

    /** Die Sonderfertigkeit die der Held erhalten/verlieren soll. */
    private final SonderfertigkeitWert sfWert;

    public AnpasserSonderfertigkeit( final Sonderfertigkeit sonderfertigkeit ){

        sfWert = new SonderfertigkeitWert( sonderfertigkeit );
    }

    public void eingefuegt( final Held held, final VorNachteilWert wert ) {

        vomBenutzerGewaehlt = held.hat( sfWert );

        if ( ! vomBenutzerGewaehlt ) {

            held.einfuegen( sfWert );
        }
    }

    public void entfernt( final Held held, final VorNachteilWert wert ) {

        if ( ! vomBenutzerGewaehlt ) {
            // Der Anpasser hat dem Helden die Sonderfertigkeit verpasst,
            // also entfernt er sie auch wieder.
            held.entfernen( sfWert );
        }
    }

    /** Macht nichts. */
    public void stufeGeaendert(final Held held, final VorNachteilWert wert) {
        // kein Implementierung noetig.
    }

    public void geaendertVariante(
            final Held held,
            final Object alteVariante,
            final Object neueVariante ) {
        // nichts zu tun.
    }

}
