/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.ArrayList;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class VoraussetzungTalent extends Voraussetzung {

    /** @return eine Liste aller Talente. */
    @Override
    public Collection<Object> getAbhaengigkeiten() {
        return new ArrayList<Object>( Talent.getTalente() );
    }

    /**
     * Prueft ob der Held das fuer den Wert gewaehlte Talent besitzt.
     */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        final Talent talent = (Talent) wert.getVariante().getElement();

        if ( ! held.hat( talent ) ) {

            beschreibung.einfuegen( I18n.i18n( "VoraussetzungBegabungTalent" ) );
        }

        return beschreibung;
    }

}
