/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

import java.util.List;

public interface Kosten {

    /**
     * Die Kostenberechnung speziell fuer den Zauber Adlerschwinge
     * Wolfsgestalt.
     */
    public static final Kosten ADLERSCHWINGE_WOLFSGESTALT =
        new KostenAdlerschwingeWolfsgestalt();

    /** Die Kostenberechnung die fuer die meisten Zauber gilt. */
    public static final Kosten STANDARD = new KostenStandard();

    /**
     * @return die Regelelemente, z.B. einen Vorteil, von dem die berechneten Kosten
     *         abhaengen.
     */
    public List<Object> getAbhaengigkeiten();

    /**
     * Liefert die Aktivierungskosten, die ein Held fuer einen Zauber aufbringen
     * muss.
     *
     * @param held
     *            der den Zauber aktivieren will.
     * @param wert
     *            der aktiviert werden soll.
     * @param lernmethode
     *            die zum Aktivieren verwendet werden soll.
     * @return die AP-Kosten, die zu zahlen sind um den Zauber zu aktivieren.
     */
    public int getAktivierungskosten(
            final Held held,
            final ZauberWert wert,
            final Lernmethode lernmethode );

    /**
     * Liefert die Steigerungsspalte fuer einen Zauber.
     *
     * @param held
     *            der den Zauber steigern will.
     * @param wert
     *            dessen Steigerungsspalte ermittelt werden soll.
     * @param stufe
     *            fuer die der Steigerungsspalte ermittelt werden soll.
     * @param lernmethode
     *            die zum Steigern verwendet werden soll.
     * @return den Steigerungsspalte, den uebergebene Wert, auf der uebergebenen
     *         Stufe hat.
     */
    public int getSteigerungsspalte(
            final Held held,
            final ZauberWert wert,
            final int stufe,
            final Lernmethode lernmethode );

    /**
     * Liefert die Steigerungskosten fuer einen Zauber.
     *
     * @param held
     *            der den Zauber steigern will.
     * @param wert
     *            fuer den die Steigerungskosten ermittelt werden sollen.
     * @param stufe
     *            von der aus der Wert um einen Punkt gesteigert werden soll.
     * @param lernmethode
     *            die zum Steigern verwendet werden soll.
     * @return Die AP-Kosten die aufzubringen sind um den Zauber um einen Punkt
     *         zu steigern.
     */
    public int getSteigerungskosten(
            final Held held,
            final ZauberWert wert,
            final int stufe,
            final Lernmethode lernmethode );
}
