/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.zauber.voraussetzungen;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

public interface Voraussetzung {

    /** Voraussetzung fuer die Zauber des Vorteils uebernatuerliche Begabung. */
    public static final Voraussetzung BEGABUNG = new VoraussetzungBegabung();

    /** Fuer Zauber die zu Beginn nur von Elfen gewaehlt werden koennen. */
    public static final Voraussetzung BEGINN_ELFEN =
        new VoraussetzungBeginnKultur( "Kultur.Auelfen",
                                       "Kultur.AuelfenHalbelf" );

    /** Fuer Zauber die zu Beginn nur von Magiern gewaehlt werden koennen. */
    public static final Voraussetzung BEGINN_MAGIER =
        new VoraussetzungBeginnProfession( "Profession.Magier" );

    /** Fuer Zauber die nur von Halbelfen gewaehlt werden koennen. */
    public static final Voraussetzung HALBELFEN =
        new VoraussetzungRasse( "Rasse.Halbelfen" );

    /**
     * Ueberprueft, dass bei der Charaktererschaffung, nur Zauber aus der
     * eigenen Repraesentation gewaehlt werden.
     */
    public static final Voraussetzung REPRAESENTATION =
        new VoraussetzungRepraesentationBeginn();

    /** Die Standardvoraussetzung fuer alle Zauber. */
    public static final Voraussetzung STANDARD = new VoraussetzungStandard();

    /** Fuer Zauber die nur fuer Viertelzauberer sind. */
    public static final Voraussetzung VIERTELZAUBERER =
        new VoraussetzungVorteil( VorNachteil.VIERTELZAUBERER,
                                  VorNachteil.VIERTELZAUBERER_UNBEWUSST );

    /** Fuer Zauber die nur fuer Vollzauberer sind. */
    public static final Voraussetzung VOLLZAUBERER =
        new VoraussetzungVorteil( VorNachteil.VOLLZAUBERER );

    /**
     * @return die Regelobjekte die fuer die zum Erfuellen der Voraussetzung wichtig
     *         sind.
     */
    public Collection< Object > getAbhaengigkeiten();

    /**
     * Ueberprueft ob ein Held den Zauber waehlen darf.
     *
     * @param held fuer den geprueft werden soll ob er den Zauber waehlen darf.
     * @param wert der gewaehlt werden soll.
     * @return Wenn der Held den Zauber nicht waehlen darf erklaert die
     *         Konfliktbeschreibung, warum.
     */
    public KonfliktBeschreibung pruefen( Held held, ZauberWert wert );
}
